import requests
import warnings
from tqdm.rich import TqdmExperimentalWarning
warnings.filterwarnings("ignore", category=TqdmExperimentalWarning)
import tarfile
import os
import shutil
import logging
import subprocess
import argparse
import sys
import re
import zipfile
from importlib.util import find_spec
from tqdm.rich import tqdm
from importlib.metadata import distributions
import configparser

# 工具版本信息
TOOL_VERSION = "hyyap -> hyy 0.0.4"

# 配置日志记录
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

# 记录已经安装过的包
installed_packages = set()

# 红色文本的 ANSI 转义序列
RED = '\033[31m'
RESET = '\033[0m'


def get_installed_packages():
    """
    获取已安装的包列表
    """
    return {dist.metadata['Name'] for dist in distributions()}


def get_package_info(package_name, version=None, mirror_url='https://pypi.org'):
    """
    从指定镜像源获取包的信息
    :param package_name: 包名
    :param version: 包的版本，可选参数
    :param mirror_url: 镜像源的 URL，默认为 PyPI
    :return: 包的信息字典
    """
    try:
        pypi_url = f'{mirror_url}/pypi/{package_name}/json'
        response = requests.get(pypi_url)
        response.raise_for_status()  # 检查请求是否成功
        package_info = response.json()

        if version:
            if version in package_info['releases']:
                package_info['info']['version'] = version
            else:
                logging.error(f"{RED}Version {version} of {package_name} is not available on {mirror_url}.{RESET}")
                return None
        return package_info
    except requests.RequestException as e:
        logging.error(f"{RED}Failed to get package information from {mirror_url}: {e}{RESET}")
        return None


def download_package(download_url, download_location='.'):
    """
    下载包文件并显示进度条
    :param download_url: 包的下载链接
    :param download_location: 下载文件保存的位置
    :return: 下载的文件名
    """
    try:
        logging.info(f"Downloading package from {download_url}")
        file_name = os.path.basename(download_url)
        full_path = os.path.join(download_location, file_name)
        response = requests.get(download_url, stream=True)
        total_size = int(response.headers.get('content-length', 0))

        # 创建下载目录（如果不存在）
        os.makedirs(download_location, exist_ok=True)

        with open(full_path, 'wb') as file, tqdm(
                desc=file_name,
                total=total_size,
                unit='B',
                unit_scale=True,
                unit_divisor=1024,
        ) as bar:
            for data in response.iter_content(chunk_size=1024):
                size = file.write(data)
                bar.update(size)

        return file_name
    except requests.RequestException as e:
        logging.error(f"{RED}Failed to download package: {e}{RESET}")
        return None


def extract_package(file_name):
    """
    解压包文件
    :param file_name: 包的文件名
    :return: 解压后的目录名
    """
    try:
        logging.info(f"Extracting package {file_name}")
        if file_name.endswith('.whl'):
            with zipfile.ZipFile(file_name, 'r') as zip_ref:
                zip_ref.extractall()
            package_dir = os.path.splitext(file_name)[0]
        elif file_name.endswith('.tar.gz'):
            with tarfile.open(file_name, 'r:gz') as tar:
                tar.extractall()
            package_dir = os.path.splitext(os.path.splitext(file_name)[0])[0]
        else:
            logging.error(f"{RED}Unsupported file format: {file_name}{RESET}")
            return None
        return package_dir
    except (tarfile.TarError, OSError, zipfile.BadZipFile) as e:
        logging.error(f"{RED}Failed to extract package: {e}{RESET}")
        return None


def print_dependencies(package_info):
    """
    打印包的依赖信息
    :param package_info: 包的信息字典
    """
    requires_dist = package_info['info'].get('requires_dist', [])
    if requires_dist:
        logging.info("Package dependencies:")
        for dep in requires_dist:
            logging.info(f"  - {dep}")
    else:
        logging.info("No dependencies found for this package.")


def install_package(package_dir):
    """
    完善的安装包函数
    :param package_dir: 解压后的包目录
    """
    if package_dir and os.path.exists(package_dir):
        try:
            site_packages_dir = os.path.join(os.path.dirname(os.__file__), 'site-packages')
            logging.info(f"Package will be installed to {site_packages_dir}")

            setup_py_path = os.path.join(package_dir, 'setup.py')
            if os.path.exists(setup_py_path):
                # 如果存在 setup.py 文件，执行安装命令
                logging.info("Found setup.py, running installation command...")
                try:
                    subprocess.run([sys.executable, setup_py_path, 'install'], check=True)
                    logging.info("Package installed successfully using setup.py")
                except subprocess.CalledProcessError as e:
                    logging.error(f"{RED}Failed to install package using setup.py: {e}{RESET}")
            else:
                # 没有 setup.py 文件，直接复制文件到 site-packages 目录
                for root, dirs, files in os.walk(package_dir):
                    relative_path = os.path.relpath(root, package_dir)
                    target_dir = os.path.join(site_packages_dir, relative_path)
                    os.makedirs(target_dir, exist_ok=True)
                    for file in files:
                        source_file = os.path.join(root, file)
                        target_file = os.path.join(target_dir, file)
                        shutil.copy2(source_file, target_file)
                logging.info("Package installed successfully by copying files")
        except (OSError, shutil.Error) as e:
            logging.error(f"{RED}Failed to install package: {e}{RESET}")
    else:
        logging.error(f"{RED}Package directory does not exist.{RESET}")


def clean_up(file_name):
    """
    清理临时文件
    :param file_name: 临时文件名
    """
    if file_name and os.path.exists(file_name):
        try:
            os.remove(file_name)
            logging.info(f"Cleaned up {file_name}")
        except OSError as e:
            logging.error(f"{RED}Failed to clean up {file_name}: {e}{RESET}")


def is_package_installed(package_name):
    """
    检测包是否已经安装
    :param package_name: 包名
    :return: 如果已安装返回 True，否则返回 False
    """
    installed = get_installed_packages()
    return package_name in installed


def handle_dependencies(package_info, mirror_url='https://pypi.org'):
    """
    处理包的依赖关系
    :param package_info: 包的信息字典
    :param mirror_url: 镜像源的 URL，默认为 PyPI
    """
    requires_dist = package_info['info'].get('requires_dist', [])
    for dep in requires_dist:
        # 使用正则表达式提取包名
        match = re.match(r'^([a-zA-Z0-9_\-]+)', dep)
        if match:
            dep_name = match.group(1)
        else:
            logging.error(f"{RED}Failed to parse dependency: {dep}{RESET}")
            continue

        if dep_name in installed_packages:
            continue

        dep_version = None
        if '(' in dep and ')' in dep:
            version_spec = dep[dep.index('(') + 1:dep.index(')')]
            # 这里简单处理，只取第一个版本限制
            if '>=' in version_spec:
                dep_version = version_spec.split('>=')[1].strip()
            elif '==' in version_spec:
                dep_version = version_spec.split('==')[1].strip()

        if not is_package_installed(dep_name):
            logging.info(f"Installing dependency: {dep_name} {dep_version or 'latest'}")
            installed_packages.add(dep_name)
            handle_install(dep_name, dep_version, mirror_url)


def handle_install(package_name, version=None, mirror_url='https://pypi.org'):
    if package_name in installed_packages:
        return

    is_installed = is_package_installed(package_name)
    if is_installed:
        logging.info(f"The package {package_name} is already installed. Skipping installation.")
        installed_packages.add(package_name)

    package_info = get_package_info(package_name, version, mirror_url)
    if not package_info:
        return

    print_dependencies(package_info)

    # 处理依赖关系
    handle_dependencies(package_info, mirror_url)

    if is_installed:
        return

    # 获取指定版本的下载链接
    target_version = package_info['info']['version']
    if target_version in package_info['releases']:
        download_url = package_info['releases'][target_version][0]['url']
    else:
        logging.error(f"{RED}Version {target_version} of {package_name} is not available on {mirror_url}.{RESET}")
        return

    file_name = download_package(download_url)
    if not file_name:
        return

    package_dir = extract_package(file_name)
    if not package_dir:
        clean_up(file_name)
        return

    install_package(package_dir)
    clean_up(file_name)
    installed_packages.add(package_name)


def handle_download(package_name, version=None, mirror_url='https://pypi.org', download_location='.', only_gz=False, gz_and_whl=False):
    package_info = get_package_info(package_name, version, mirror_url)
    if not package_info:
        return

    if not version:
        # 如果没有指定版本，使用最新版本
        version = package_info['info']['version']

    # 获取指定版本的下载链接
    if version in package_info['releases']:
        releases = package_info['releases'][version]
        if only_gz:
            gz_links = [rel['url'] for rel in releases if rel['filename'].endswith('.tar.gz')]
            if not gz_links:
                logging.error(f"{RED}No .gz files available for {package_name} version {version} on {mirror_url}.{RESET}")
                return
            for link in gz_links:
                file_name = download_package(link, download_location)
                if file_name:
                    logging.info(f"Package downloaded successfully to {download_location} as {file_name}")
        elif gz_and_whl:
            gz_links = [rel['url'] for rel in releases if rel['filename'].endswith('.tar.gz')]
            whl_links = [rel['url'] for rel in releases if rel['filename'].endswith('.whl')]
            if not gz_links:
                logging.error(f"{RED}No .gz files available for {package_name} version {version} on {mirror_url}.{RESET}")
            else:
                for link in gz_links:
                    file_name = download_package(link, download_location)
                    if file_name:
                        logging.info(f"Package downloaded successfully to {download_location} as {file_name}")
            if not whl_links:
                logging.error(f"{RED}No .whl files available for {package_name} version {version} on {mirror_url}.{RESET}")
            else:
                for link in whl_links:
                    file_name = download_package(link, download_location)
                    if file_name:
                        logging.info(f"Package downloaded successfully to {download_location} as {file_name}")
        else:
            # 正常下载逻辑
            download_url = releases[0]['url']
            file_name = download_package(download_url, download_location)
            if file_name:
                logging.info(f"Package downloaded successfully to {download_location} as {file_name}")
    else:
        logging.error(f"{RED}Version {version} of {package_name} is not available on {mirror_url}.{RESET}")


def uninstall_package(package_name):
    """
    卸载包
    :param package_name: 包名
    """
    if not is_package_installed(package_name):
        logging.info(f"The package {package_name} is not installed. Skipping uninstallation.")
        return

    site_packages_dir = os.path.join(os.path.dirname(os.__file__), 'site-packages')
    package_dir = os.path.join(site_packages_dir, package_name)
    egg_info_dir = None
    for item in os.listdir(site_packages_dir):
        if item.endswith('.egg-info') and item.startswith(package_name):
            egg_info_dir = os.path.join(site_packages_dir, item)
            break

    try:
        if os.path.exists(package_dir):
            shutil.rmtree(package_dir)
            logging.info(f"Removed package directory: {package_dir}")
        if egg_info_dir:
            shutil.rmtree(egg_info_dir)
            logging.info(f"Removed egg-info directory: {egg_info_dir}")
        logging.info(f"Package {package_name} uninstalled successfully.")
    except (OSError, shutil.Error) as e:
        logging.error(f"{RED}Failed to uninstall package {package_name}: {e}{RESET}")


def list_installed_packages():
    """
    列出所有已安装的模块
    """
    logging.info("List of installed packages:")
    for dist in distributions():
        logging.info(f"{dist.metadata['Name']}=={dist.metadata['Version']}")


def handle_update(package_name, mirror_url='https://pypi.org'):
    """
    更新包
    :param package_name: 包名
    :param mirror_url: 镜像源的 URL，默认为 PyPI
    """
    if not is_package_installed(package_name):
        logging.info(f"The package {package_name} is not installed. Skipping update.")
        return
    uninstall_package(package_name)
    handle_install(package_name, mirror_url=mirror_url)


def set_permanent_mirror(mirror_url):
    """
    设置永久的镜像源
    :param mirror_url: 镜像源的 URL
    """
    config = configparser.ConfigParser()
    if os.name == 'nt':  # Windows
        config_dir = os.path.join(os.getenv('APPDATA'), 'your_package_manager')
        config_file = os.path.join(config_dir, 'config.ini')
    else:  # Linux/Mac
        config_dir = os.path.join(os.path.expanduser('~'), '.your_package_manager')
        config_file = os.path.join(config_dir, 'config.ini')

    os.makedirs(config_dir, exist_ok=True)

    if not config.has_section('mirror'):
        config.add_section('mirror')
    config.set('mirror', 'url', mirror_url)

    with open(config_file, 'w') as f:
        config.write(f)

    logging.info(f"Permanent mirror set to {mirror_url}")


def get_permanent_mirror():
    """
    获取永久设置的镜像源
    :return: 镜像源的 URL，如果未设置则返回默认值
    """
    config = configparser.ConfigParser()
    if os.name == 'nt':  # Windows
        config_dir = os.path.join(os.getenv('APPDATA'), 'your_package_manager')
        config_file = os.path.join(config_dir, 'config.ini')
    else:  # Linux/Mac
        config_dir = os.path.join(os.path.expanduser('~'), '.your_package_manager')
        config_file = os.path.join(config_dir, 'config.ini')

    if os.path.exists(config_file):
        config.read(config_file)
        if config.has_section('mirror') and config.has_option('mirror', 'url'):
            return config.get('mirror', 'url')
    return 'https://pypi.org'


def main():
    parser = argparse.ArgumentParser(description='Simple Python package manager')
    parser.add_argument('--version', action='version', version=f'%(prog)s {TOOL_VERSION}')
    subparsers = parser.add_subparsers(dest='command', help='Available commands')

    # install 命令
    install_parser = subparsers.add_parser('install', help='Install a Python package')
    install_parser.add_argument('package_name', help='Name of the package to install')
    install_parser.add_argument('-v', '--package-version', help='Version of the package to install')
    install_parser.add_argument('-m', '--mirror-url', help='Mirror URL to use, will override permanent setting')

    # download 命令
    download_parser = subparsers.add_parser('download', help='Download a Python package')
    download_parser.add_argument('package_name', help='Name of the package to download')
    download_parser.add_argument('-v', '--package-version', help='Version of the package to download')
    download_parser.add_argument('-m', '--mirror-url', help='Mirror URL to use, will override permanent setting')
    download_parser.add_argument('-d', '--download-location', default='.', help='Location to save the downloaded package')
    download_parser.add_argument('-g', action='store_true', help='Only download .gz files')
    download_parser.add_argument('-gw', action='store_true', help='Download both .gz and .whl files')

    # uninstall 命令
    uninstall_parser = subparsers.add_parser('uninstall', help='Uninstall a Python package')
    uninstall_parser.add_argument('package_name', help='Name of the package to uninstall')

    # list 命令
    list_parser = subparsers.add_parser('list', help='List all installed Python packages')

    # update 命令
    update_parser = subparsers.add_parser('update', help='Update a Python package')
    update_parser.add_argument('package_name', help='Name of the package to update')
    update_parser.add_argument('-m', '--mirror-url', help='Mirror URL to use, will override permanent setting')

    # set-mirror 命令
    set_mirror_parser = subparsers.add_parser('set-mirror', help='Set a permanent mirror URL')
    set_mirror_parser.add_argument('mirror_url', help='Mirror URL to set as permanent')

    args = parser.parse_args()

    if args.command == 'install':
        mirror_url = args.mirror_url if args.mirror_url else get_permanent_mirror()
        logging.info(f"Using mirror: {mirror_url}")
        try:
            handle_install(args.package_name, args.package_version, mirror_url)
        except Exception as e:
            logging.error(f"{RED}Installation failed: {e}{RESET}")
    elif args.command == 'download':
        mirror_url = args.mirror_url if args.mirror_url else get_permanent_mirror()
        download_location = args.download_location
        only_gz = args.g
        gz_and_whl = args.gw
        logging.info(f"Using mirror: {mirror_url}")
        try:
            handle_download(args.package_name, args.package_version, mirror_url, download_location, only_gz, gz_and_whl)
        except Exception as e:
            logging.error(f"{RED}Download failed: {e}{RESET}")

    elif args.command == 'uninstall':
        try:
            uninstall_package(args.package_name)
        except Exception as e:
            logging.error(f"{RED}Uninstallation failed: {e}{RESET}")
    elif args.command == 'list':
        list_installed_packages()
    elif args.command == 'update':
        mirror_url = args.mirror_url if args.mirror_url else get_permanent_mirror()
        logging.info(f"Using mirror: {mirror_url}")
        try:
            handle_update(args.package_name, mirror_url)
        except Exception as e:
            logging.error(f"{RED}Update failed: {e}{RESET}")
    elif args.command == 'set-mirror':
        set_permanent_mirror(args.mirror_url)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()


import os
import requests
from tqdm import tqdm
from rich.progress import Progress, TransferSpeedColumn, DownloadColumn
from rich.console import Console
import py7zr
import winreg

def download_and_extract_mingw(arch='i686', show_progress=True):
    """
    下载并解压 MinGW 的 7z 文件

    :param arch: MinGW 的架构，可选 'i686' 或 'x86_64'
    :param show_progress: 是否显示下载进度条
    :return: 解压后的文件夹路径
    """
    if arch == 'i686':
        url = "https://objects.githubusercontent.com/github-production-release-asset-2e65be/446033510/743a1268-21b9-4041-8f7f-8c0ac84c5843?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=releaseassetproduction%2F20250225%2Fus-east-1%2Fs3%2Faws4_request&X-Amz-Date=20250225T160901Z&X-Amz-Expires=300&X-Amz-Signature=5b0c3af7c200844871473d3f5b457bc39b5cbcc8695e58ddcd5ef85ebbac881d&X-Amz-SignedHeaders=host&response-content-disposition=attachment%3B%20filename%3Di686-14.2.0-release-mcf-dwarf-ucrt-rt_v12-rev1.7z&response-content-type=application%2Foctet-stream"
        save_path = os.path.join(os.path.expanduser("~"), "i686-14.2.0-release-mcf-dwarf-ucrt-rt_v12-rev1.7z")
        extract_dir = os.path.join(os.path.expanduser("~"), "mingw64")
        new_name = "i686mingw64"
    else:
        url = "https://objects.githubusercontent.com/github-production-release-asset-2e65be/446033510/aba497ad-caca-4f91-8aa8-d62e183feab9?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=releaseassetproduction%2F20250225%2Fus-east-1%2Fs3%2Faws4_request&X-Amz-Date=20250225T162851Z&X-Amz-Expires=300&X-Amz-Signature=bb3067eb988aa64c8af3c247ddf48b24de21b13b429653db1095738552e0f082&X-Amz-SignedHeaders=host&response-content-disposition=attachment%3B%20filename%3Dx86_64-14.2.0-release-mcf-seh-ucrt-rt_v12-rev1.7z&response-content-type=application%2Foctet-stream"
        save_path = os.path.join(os.path.expanduser("~"), "x86_64-14.2.0-release-mcf-seh-ucrt-rt_v12-rev1.7z")
        extract_dir = os.path.join(os.path.expanduser("~"), "mingw64")
        new_name = "x86_64mingw64"

    # 下载文件
    if show_progress:
        console = Console()
        with Progress(
            "[progress.description]{task.description}",
            TransferSpeedColumn(),
            DownloadColumn(),
            console=console,
            expand=True
        ) as progress:
            task = progress.add_task("[cyan]Downloading...", total=None)
            response = requests.get(url, stream=True)
            total_size = int(response.headers.get('content-length', 0))
            progress.update(task, total=total_size)
            with open(save_path, 'wb') as file:
                for data in tqdm(response.iter_content(chunk_size=1024), total=total_size // 1024, unit='KB'):
                    file.write(data)
                    progress.update(task, advance=len(data))
            progress.update(task, description="[green]Download completed!")
    else:
        response = requests.get(url)
        with open(save_path, 'wb') as file:
            file.write(response.content)

    # 解压文件
    with py7zr.SevenZipFile(save_path, mode='r') as z:
        z.extractall(path=extract_dir)

    # 重命名文件夹
    new_extract_dir = os.path.join(os.path.dirname(extract_dir), new_name)
    if os.path.exists(extract_dir):
        os.rename(extract_dir, new_extract_dir)

    # 删除 7z 文件
    if os.path.exists(save_path):
        os.remove(save_path)

    return new_extract_dir

def set_mingw_environment_variable(mingw_bin_path):
    """
    设置 MinGW 的环境变量

    :param mingw_bin_path: MinGW 的 bin 文件夹路径
    """
    # 打开注册表项
    key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r'Environment', 0, winreg.KEY_ALL_ACCESS)

    try:
        # 获取当前的 PATH 环境变量
        path_var, _ = winreg.QueryValueEx(key, 'PATH')
        paths = path_var.split(';')

        # 移除包含 \mingw64 或 \mingw64\bin 的路径
        new_paths = [p for p in paths if not ('\\mingw64' in p or '\\mingw64\\bin' in p)]

        # 添加新的 mingw64\bin 路径
        if mingw_bin_path not in new_paths:
            new_paths.append(mingw_bin_path)

        # 更新 PATH 环境变量
        new_path_var = ';'.join(new_paths)
        winreg.SetValueEx(key, 'PATH', 0, winreg.REG_EXPAND_SZ, new_path_var)
    finally:
        # 关闭注册表项
        winreg.CloseKey(key)

class MinGWInstaller:
    """show_progress: 是否开启进度条"""
    def __init__(self,show_progress = False):
        self.show_progress = show_progress

    def i686(self):
        """show_progress: 是否开启进度条"""
        arch = 'i686'
        mingw_dir = download_and_extract_mingw(arch, self.show_progress)
        mingw_bin_path = os.path.join(mingw_dir, "bin")
        set_mingw_environment_variable(mingw_bin_path)


    def x86_64(self):
        """show_progress: 是否开启进度条"""
        arch = 'x86_64'
        mingw_dir = download_and_extract_mingw(arch, self.show_progress)
        mingw_bin_path = os.path.join(mingw_dir, "bin")
        set_mingw_environment_variable(mingw_bin_path)


from .apopen import HPopen, _Ap_Popen
import time


def run(command, shell=False):
    """
    在 cmd 或 shell 中执行命令
    :param command: 要执行的命令，以列表形式传入
    :param shell: 是否使用 shell 执行命令，默认为 False
    :return: CommunicationResult 对象，包含标准输出、标准错误和退出状态码
    """
    process = _Ap_Popen(command, shell=shell)
    return process.communicate()
    

class RunTime:
    _instance = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
            cls._instance.start = time.time()
        return cls._instance

    @property
    def time(self):
        return time.time() - self.start
    
Runtime = RunTime()


import sys
import threading
import platform
import subprocess
from .shared import CommunicationResult

class HPopen:
    def __init__(self, args, shell=False, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr):
        if shell:
            if platform.system() == 'Windows':
                shell_cmd = ['cmd', '/c'] + args
            else:
                shell_cmd = ['sh', '-c'] + args
            args = shell_cmd

        self.args = args
        self.stdout_data = []
        self.stderr_data = []
        self.process = None
        self._start_process()

    def _start_process(self):
        try:
            self.process = subprocess.Popen(
                self.args,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                shell=False
            )

            self.stdout_thread = threading.Thread(target=self._read_output, args=(self.process.stdout, self.stdout_data))
            self.stderr_thread = threading.Thread(target=self._read_output, args=(self.process.stderr, self.stderr_data))
            self.stdout_thread.start()
            self.stderr_thread.start()
        except Exception as e:
            print(f"Error starting process: {e}", file=sys.stderr)

    def _read_output(self, stream, data_list):
        try:
            encoding = 'gbk' if platform.system() == 'Windows' else 'utf-8'
            for line in iter(stream.readline, b''):
                try:
                    decoded_line = line.decode(encoding)
                except UnicodeDecodeError:
                    try:
                        decoded_line = line.decode('utf-8', errors='replace')
                    except UnicodeDecodeError:
                        continue
                data_list.append(decoded_line)
        except Exception as e:
            print(f"Error reading output: {e}", file=sys.stderr)
        finally:
            stream.close()

    def communicate(self, input_data=None):
        if self.process is None:
            return CommunicationResult("", f"Process failed to start: {self.args}", 1)

        if input_data:
            if isinstance(input_data, str):
                input_data = input_data.encode()
            self.process.stdin.write(input_data)
            self.process.stdin.flush()
        if self.process.stdin:
            self.process.stdin.close()

        if self.stdout_thread:
            self.stdout_thread.join()
        if self.stderr_thread:
            self.stderr_thread.join()

        if self.process:
            self.process.wait()
            status = self.process.returncode
            stdout_str = ''.join(self.stdout_data)
            stderr_str = ''.join(self.stderr_data)
            return CommunicationResult(stdout_str, stderr_str, status)
        return CommunicationResult("", "Process is None", 1)

    def wait(self):
        if self.process:
            self.process.wait()
            return self.process.returncode
        return 1


class _Ap_Popen:
    def __init__(self, args, shell=False, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr):
        if shell:
            if platform.system() == 'Windows':
                shell_cmd = ['cmd', '/c'] + args
            else:
                shell_cmd = ['sh', '-c'] + args
            args = shell_cmd

        self.args = args
        self.stdout_data = []
        self.stderr_data = []
        self.process = None
        self._start_process()

    def _start_process(self):
        try:
            self.process = subprocess.Popen(
                self.args,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                shell=False
            )

            self.stdout_thread = threading.Thread(target=self._read_output, args=(self.process.stdout, self.stdout_data))
            self.stderr_thread = threading.Thread(target=self._read_output, args=(self.process.stderr, self.stderr_data))
            self.stdout_thread.start()
            self.stderr_thread.start()
        except Exception as e:
            print(f"Error starting process: {e}", file=sys.stderr)

    def _read_output(self, stream, data_list):
        try:
            encoding = 'gbk' if platform.system() == 'Windows' else 'utf-8'
            for line in iter(stream.readline, b''):
                try:
                    decoded_line = line.decode(encoding)
                except UnicodeDecodeError:
                    try:
                        decoded_line = line.decode('utf-8', errors='replace')
                    except UnicodeDecodeError:
                        continue
                data_list.append(decoded_line)
        except Exception as e:
            print(f"Error reading output: {e}", file=sys.stderr)
        finally:
            stream.close()

    def communicate(self, input_data=None):
        if self.process is None:
            return CommunicationResult("", f"Process failed to start: {self.args}", 1)

        if input_data:
            if isinstance(input_data, str):
                input_data = input_data.encode()
            self.process.stdin.write(input_data)
            self.process.stdin.flush()
        if self.process.stdin:
            self.process.stdin.close()

        if self.stdout_thread:
            self.stdout_thread.join()
        if self.stderr_thread:
            self.stderr_thread.join()

        if self.process:
            self.process.wait()
            status = self.process.returncode
            stdout_str = ''.join(self.stdout_data)
            stderr_str = ''.join(self.stderr_data)
            return CommunicationResult(stdout_str, stderr_str, status)
        return CommunicationResult("", "Process is None", 1)

    def wait(self):
        if self.process:
            self.process.wait()
            return self.process.returncode
        return 1


import xml.etree.ElementTree as ET
from bs4 import BeautifulSoup
import html
import requests


class Xmlhtml:
    def __init__(self, xml_content=None, html_content=None):
        """
        初始化类，可传入 XML 或 HTML 内容
        :param xml_content: XML 内容
        :param html_content: HTML 内容
        """
        self.xml_content = xml_content
        self.html_content = html_content
        if self.xml_content:
            self.xml_root = ET.fromstring(self.xml_content)
        else:
            self.xml_root = None

    def get_xml_info(self):
        """
        从 XML 中提取信息并返回
        :return: 包含 XML 信息的列表
        """
        if not self.xml_content:
            return []
        info = []

        def traverse(element):
            info.append((element.tag, element.text))
            for child in element:
                traverse(child)

        traverse(self.xml_root)
        return info

    def xml_to_html(self):
        """
        将 XML 转换为 HTML
        :return: 转换后的 HTML 内容
        """
        if not self.xml_content:
            return ""
        html_parts = []

        def build_html(element):
            tag = element.tag
            text = element.text or ""
            html_parts.append(f"<{tag}>")
            html_parts.append(html.escape(text))
            for child in element:
                build_html(child)
            html_parts.append(f"</{tag}>")

        build_html(self.xml_root)
        return ''.join(html_parts)

    def html_to_xml(self):
        """
        将 HTML 转换为 XML
        :return: 转换后的 XML 内容
        """
        if not self.html_content:
            return ""
        soup = BeautifulSoup(self.html_content, 'html.parser')
        xml_parts = []

        def build_xml(element):
            tag = element.name
            text = element.get_text()
            xml_parts.append(f"<{tag}>")
            xml_parts.append(html.escape(text))
            for child in element.children:
                if isinstance(child, BeautifulSoup):
                    continue
                build_xml(child)
            xml_parts.append(f"</{tag}>")

        build_xml(soup)
        return ''.join(xml_parts)

    def find_elements_by_tag(self, tag):
        """
        根据标签名查找 XML 中的元素
        :param tag: 要查找的标签名
        :return: 包含匹配元素的列表
        """
        if not self.xml_content:
            return []
        return self.xml_root.findall(f'.//{tag}')

    def modify_element_text(self, tag, new_text):
        """
        修改指定标签元素的文本内容
        :param tag: 要修改的标签名
        :param new_text: 新的文本内容
        :return: 修改后的 XML 字符串
        """
        if not self.xml_content:
            return ""
        elements = self.xml_root.findall(f'.//{tag}')
        for element in elements:
            element.text = new_text
        self.xml_content = ET.tostring(self.xml_root, encoding='unicode')
        return self.xml_content

    def delete_elements_by_tag(self, tag):
        """
        根据标签名删除 XML 中的元素
        :param tag: 要删除的标签名
        :return: 删除元素后的 XML 字符串
        """
        if not self.xml_content:
            return ""
        for parent in self.xml_root.findall('.//*'):
            for element in parent.findall(tag):
                parent.remove(element)
        self.xml_content = ET.tostring(self.xml_root, encoding='unicode')
        return self.xml_content

    def save_xml(self, file_path):
        """
        将 XML 数据保存到指定文件
        :param file_path: 保存文件的路径
        """
        if self.xml_content:
            with open(file_path, 'w', encoding='utf-8') as file:
                file.write(self.xml_content)
        else:
            print("没有可用的 XML 数据进行保存。")

    def load_xml(self, file_path):
        """
        从指定文件加载 XML 数据
        :param file_path: 加载文件的路径
        """
        try:
            with open(file_path, 'r', encoding='utf-8') as file:
                self.xml_content = file.read()
                self.xml_root = ET.fromstring(self.xml_content)
        except FileNotFoundError:
            print(f"文件 {file_path} 未找到。")
        except ET.ParseError:
            print(f"无法解析 {file_path} 中的 XML 数据。")

    def load_xml_from_url(self, url):
        """
        从指定 URL 加载 XML 文件
        :param url: XML 文件的 URL
        """
        try:
            response = requests.get(url)
            response.raise_for_status()
            self.xml_content = response.text
            self.xml_root = ET.fromstring(self.xml_content)
        except requests.RequestException as e:
            print(f"请求 URL {url} 时出现错误: {e}")
        except ET.ParseError:
            print(f"无法解析从 {url} 获取的 XML 数据。")

    def load_html_from_url(self, url):
        """
        从指定 URL 加载 HTML 文件
        :param url: HTML 文件的 URL
        """
        try:
            response = requests.get(url)
            response.raise_for_status()
            self.html_content = response.text
        except requests.RequestException as e:
            print(f"请求 URL {url} 时出现错误: {e}")




import psutil
import platform
import pynvml
import subprocess
import wmi
import win32api,win32con,os


class CPUInfo:
    def __init__(self):
        pass

    def get_physical_core_count(self):
        return psutil.cpu_count(logical=False)

    def get_logical_core_count(self):
        return psutil.cpu_count(logical=True)

    def get_cpu_percent(self, interval=1):
        return psutil.cpu_percent(interval=interval)


class MemoryInfo:
    def __init__(self):
        pass

    def get_total_memory(self):
        memory = psutil.virtual_memory()
        return memory.total / (1024 ** 3)

    def get_used_memory(self):
        memory = psutil.virtual_memory()
        return memory.used / (1024 ** 3)

    def get_memory_percent(self):
        memory = psutil.virtual_memory()
        return memory.percent


class DiskInfo:
    def __init__(self, path='/'):
        self.path = path

    def get_total_disk_space(self):
        disk = psutil.disk_usage(self.path)
        return disk.total / (1024 ** 3)

    def get_used_disk_space(self):
        disk = psutil.disk_usage(self.path)
        return disk.used / (1024 ** 3)

    def get_disk_percent(self):
        disk = psutil.disk_usage(self.path)
        return disk.percent


class GPUInfo:
    def __init__(self):
        self.is_windows = platform.system() == 'Windows'
        if self.is_windows:
            self.w = wmi.WMI()
        try:
            pynvml.nvmlInit()
            self.has_nvidia_gpu = True
        except pynvml.NVMLError:
            self.has_nvidia_gpu = False

    def get_gpu_names(self):
        if self.is_windows:
            gpu_names = []
            for gpu in self.w.Win32_VideoController():
                gpu_names.append(gpu.Name)
            return gpu_names
        return []

    def get_nvidia_gpu_memory_info(self):
        if self.has_nvidia_gpu:
            device_count = pynvml.nvmlDeviceGetCount()
            gpu_memory_info = []
            for i in range(device_count):
                handle = pynvml.nvmlDeviceGetHandleByIndex(i)
                info = pynvml.nvmlDeviceGetMemoryInfo(handle)
                total_memory = info.total / (1024 ** 3)
                used_memory = info.used / (1024 ** 3)
                memory_percent = (used_memory / total_memory) * 100
                gpu_memory_info.append({
                    'total': total_memory,
                    'used': used_memory,
                    'percent': memory_percent
                })
            return gpu_memory_info
        return []

    def __del__(self):
        if self.has_nvidia_gpu:
            pynvml.nvmlShutdown()


class MotherboardInfo:
    def __init__(self):
        self.w = wmi.WMI()

    def get_motherboard_manufacturer(self):
        """获取主板制造商信息"""
        for board in self.w.Win32_BaseBoard():
            return board.Manufacturer
        return None

    def get_motherboard_product(self):
        """获取主板产品型号信息"""
        for board in self.w.Win32_BaseBoard():
            return board.Product
        return None

    def get_motherboard_serial_number(self):
        """获取主板序列号信息"""
        for board in self.w.Win32_BaseBoard():
            return board.SerialNumber
        return None

class NetworkAdapterInfo:
    def __init__(self):
        self.system = platform.system()
        if self.system == 'Windows':
            self.w = wmi.WMI()

    def get_network_adapter_names(self):
        if self.system == 'Windows':
            adapter_names = []
            for nic in self.w.Win32_NetworkAdapter():
                adapter_names.append(nic.Name)
            return adapter_names
        elif self.system == 'Linux':
            return list(psutil.net_if_addrs().keys())
        return []

    def get_network_adapter_mac_addresses(self):
        if self.system == 'Windows':
            mac_addresses = []
            for nic in self.w.Win32_NetworkAdapterConfiguration(IPEnabled=1):
                mac_addresses.append(nic.MACAddress)
            return mac_addresses
        elif self.system == 'Linux':
            mac_addresses = {}
            for interface, addrs in psutil.net_if_addrs().items():
                for addr in addrs:
                    if addr.family == psutil.AF_LINK:
                        mac_addresses[interface] = addr.address
            return mac_addresses
        return []

class ScreenInfo:
    def __init__(self):
        self.system = platform.system()

    def get_screen_count(self):
        if self.system == 'Windows':
            return win32api.GetSystemMetrics(win32con.SM_CMONITORS)
        elif self.system == 'Linux':
            try:
                output = subprocess.check_output(['xrandr', '--listmonitors']).decode('utf-8')
                lines = output.strip().split('\n')
                return len(lines) - 1
            except subprocess.CalledProcessError:
                return 0
        return 0

    def get_screen_resolutions(self):
        resolutions = []
        if self.system == 'Windows':
            for i in range(self.get_screen_count()):
                monitor_info = win32api.EnumDisplayMonitors()[i]
                monitor = win32api.GetMonitorInfo(monitor_info[0])
                left, top, right, bottom = monitor["Monitor"]
                width = right - left
                height = bottom - top
                resolutions.append((width, height))
        elif self.system == 'Linux':
            try:
                output = subprocess.check_output(['xrandr']).decode('utf-8')
                lines = output.strip().split('\n')
                for line in lines:
                    if ' connected ' in line:
                        parts = line.split()
                        for part in parts:
                            if 'x' in part:
                                try:
                                    width, height = map(int, part.split('x'))
                                    resolutions.append((width, height))
                                    break
                                except ValueError:
                                    continue
            except subprocess.CalledProcessError:
                pass
        return resolutions

import cv2

class HardwareInfo:
    def __init__(self):
        self.system = platform.system()
        if self.system == 'Windows':
            self.w = wmi.WMI()

    def get_sound_card_info(self):
        if self.system == 'Windows':
            sound_cards = []
            for sound_card in self.w.Win32_SoundDevice():
                sound_cards.append({
                    'Name': sound_card.Name,
                    'Manufacturer': sound_card.Manufacturer
                })
            return sound_cards
        elif self.system == 'Linux':
            sound_cards = []
            if os.path.exists('/proc/asound/cards'):
                with open('/proc/asound/cards', 'r') as f:
                    lines = f.readlines()
                    card_info = {}
                    for line in lines:
                        if line.startswith(' '):
                            parts = line.strip().split(':')
                            if len(parts) == 2:
                                key = parts[0].strip()
                                value = parts[1].strip()
                                if key == 'Card':
                                    card_info['Name'] = value
                                elif key == 'Driver':
                                    card_info['Driver'] = value
                        elif line.strip() == '':
                            if card_info:
                                sound_cards.append(card_info)
                                card_info = {}
                    if card_info:
                        sound_cards.append(card_info)
            return sound_cards
        return []

    def get_camera_info(self):
        if self.system == 'Windows':
            camera_indexes = []
            index = 0
            while True:
                cap = cv2.VideoCapture(index)
                if not cap.read()[0]:
                    break
                else:
                    camera_indexes.append(index)
                cap.release()
                index += 1
            return camera_indexes
        elif self.system == 'Linux':
            cameras = []
            for i in range(10):
                device_path = f'/dev/video{i}'
                if os.path.exists(device_path):
                    cameras.append(device_path)
            return cameras
        return []


demoy1 = """# CPU 信息
cpu = CPUInfo()
print(f"物理核心数: {cpu.get_physical_core_count()}")
print(f"逻辑核心数: {cpu.get_logical_core_count()}")
print(f"CPU 使用率: {cpu.get_cpu_percent()}%")

# 内存信息
memory = MemoryInfo()
print(f"总内存: {memory.get_total_memory():.2f} GB")
print(f"已使用内存: {memory.get_used_memory():.2f} GB")
print(f"内存使用率: {memory.get_memory_percent()}%")

# 磁盘信息
disk = DiskInfo()
print(f"总磁盘空间: {disk.get_total_disk_space():.2f} GB")
print(f"已使用磁盘空间: {disk.get_used_disk_space():.2f} GB")
print(f"磁盘使用率: {disk.get_disk_percent()}%")

# 显卡和显存信息
gpu = GPUInfo()
print(f"显卡名称: {gpu.get_gpu_names()}")
nvidia_gpu_memory_info = gpu.get_nvidia_gpu_memory_info()
for i, info in enumerate(nvidia_gpu_memory_info):
    print(f"NVIDIA GPU {i} 总显存: {info['total']:.2f} GB")
    print(f"NVIDIA GPU {i} 已使用显存: {info['used']:.2f} GB")
    print(f"NVIDIA GPU {i} 显存使用率: {info['percent']:.2f}%")
    
motherboard = MotherboardInfo()
print(f"主板制造商: {motherboard.get_motherboard_manufacturer()}")
print(f"主板产品型号: {motherboard.get_motherboard_product()}")
print(f"主板序列号: {motherboard.get_motherboard_serial_number()}")

network_info = NetworkAdapterInfo()
print("网卡名称:", network_info.get_network_adapter_names())
print("网卡 MAC 地址:", network_info.get_network_adapter_mac_addresses())

screen = ScreenInfo()
print(f"屏幕数量: {screen.get_screen_count()}")
print(f"屏幕分辨率: {screen.get_screen_resolutions()}")

hardware_info = HardwareInfo()
print("声卡信息:", hardware_info.get_sound_card_info())
print("摄像头信息:", hardware_info.get_camera_info())"""

import socket

class System:
    def __init__(self):
        self.system = platform.system()
        if self.system == 'Windows':
            self.w = wmi.WMI()

    def get_os_info(self):
        """获取操作系统信息"""
        os_name = platform.system()
        os_version = platform.version()
        return {
            'name': os_name,
            'version': os_version
        }

    def get_cpu_info(self):
        """获取 CPU 信息"""
        cpu_count = psutil.cpu_count(logical=False)  # 物理核心数
        cpu_count_logical = psutil.cpu_count(logical=True)  # 逻辑核心数
        cpu_percent = psutil.cpu_percent(interval=1)  # CPU 使用率
        return {
            'physical_cores': cpu_count,
            'logical_cores': cpu_count_logical,
            'usage_percent': cpu_percent
        }

    def get_memory_info(self):
        """获取内存信息"""
        memory = psutil.virtual_memory()
        total_memory = memory.total / (1024 ** 3)  # 总内存（GB）
        used_memory = memory.used / (1024 ** 3)  # 已使用内存（GB）
        memory_percent = memory.percent  # 内存使用率
        return {
            'total_gb': total_memory,
            'used_gb': used_memory,
            'usage_percent': memory_percent
        }

    def get_disk_info(self, path='/'):
        """获取磁盘信息"""
        disk = psutil.disk_usage(path)
        total_disk = disk.total / (1024 ** 3)  # 总磁盘空间（GB）
        used_disk = disk.used / (1024 ** 3)  # 已使用磁盘空间（GB）
        disk_percent = disk.percent  # 磁盘使用率
        return {
            'total_gb': total_disk,
            'used_gb': used_disk,
            'usage_percent': disk_percent
        }

    def get_network_info(self):
        """获取网络信息"""
        network_info = psutil.net_if_addrs()
        network_stats = psutil.net_io_counters()
        interfaces = []
        for interface, addrs in network_info.items():
            interface_info = {
                'name': interface,
                'addresses': []
            }
            for addr in addrs:
                if addr.family == socket.AF_INET:
                    interface_info['addresses'].append({
                        'family': 'IPv4',
                        'address': addr.address,
                        'netmask': addr.netmask
                    })
                elif addr.family == socket.AF_INET6:
                    interface_info['addresses'].append({
                        'family': 'IPv6',
                        'address': addr.address,
                        'netmask': addr.netmask
                    })
            interfaces.append(interface_info)

        return {
            'interfaces': interfaces,
            'bytes_sent': network_stats.bytes_sent,
            'bytes_recv': network_stats.bytes_recv
        }

    def get_motherboard_info(self):
        """获取主板信息"""
        if self.system == 'Windows':
            for board in self.w.Win32_BaseBoard():
                return {
                    'manufacturer': board.Manufacturer,
                    'product': board.Product,
                    'serial_number': board.SerialNumber
                }
        elif self.system == 'Linux':
            motherboard_info = {}
            board_vendor_path = '/sys/class/dmi/id/board_vendor'
            board_name_path = '/sys/class/dmi/id/board_name'
            board_serial_path = '/sys/class/dmi/id/board_serial'

            if os.path.exists(board_vendor_path):
                with open(board_vendor_path, 'r') as f:
                    motherboard_info['manufacturer'] = f.read().strip()
            if os.path.exists(board_name_path):
                with open(board_name_path, 'r') as f:
                    motherboard_info['product'] = f.read().strip()
            if os.path.exists(board_serial_path):
                with open(board_serial_path, 'r') as f:
                    motherboard_info['serial_number'] = f.read().strip()
            return motherboard_info
        return {}

    def get_gpu_info(self):
        """获取显卡信息"""
        if self.system == 'Windows':
            gpu_names = []
            for gpu in self.w.Win32_VideoController():
                gpu_names.append(gpu.Name)
            return {
                'names': gpu_names
            }
        elif self.system == 'Linux':
            try:
                output = subprocess.check_output(['lspci']).decode('utf-8')
                gpu_names = []
                for line in output.splitlines():
                    if 'VGA compatible controller' in line:
                        parts = line.split(':')
                        gpu_names.append(parts[-1].strip())
                return {
                    'names': gpu_names
                }
            except subprocess.CalledProcessError:
                return {}
        return {}

def demoy():
    # CPU 信息
    cpu = CPUInfo()
    print(f"物理核心数: {cpu.get_physical_core_count()}")
    print(f"逻辑核心数: {cpu.get_logical_core_count()}")
    print(f"CPU 使用率: {cpu.get_cpu_percent()}%")

    # 内存信息
    memory = MemoryInfo()
    print(f"总内存: {memory.get_total_memory():.2f} GB")
    print(f"已使用内存: {memory.get_used_memory():.2f} GB")
    print(f"内存使用率: {memory.get_memory_percent()}%")

    # 磁盘信息
    disk = DiskInfo()
    print(f"总磁盘空间: {disk.get_total_disk_space():.2f} GB")
    print(f"已使用磁盘空间: {disk.get_used_disk_space():.2f} GB")
    print(f"磁盘使用率: {disk.get_disk_percent()}%")

    # 显卡和显存信息
    gpu = GPUInfo()
    print(f"显卡名称: {gpu.get_gpu_names()}")
    nvidia_gpu_memory_info = gpu.get_nvidia_gpu_memory_info()
    for i, info in enumerate(nvidia_gpu_memory_info):
        print(f"NVIDIA GPU {i} 总显存: {info['total']:.2f} GB")
        print(f"NVIDIA GPU {i} 已使用显存: {info['used']:.2f} GB")
        print(f"NVIDIA GPU {i} 显存使用率: {info['percent']:.2f}%")

    motherboard = MotherboardInfo()
    print(f"主板制造商: {motherboard.get_motherboard_manufacturer()}")
    print(f"主板产品型号: {motherboard.get_motherboard_product()}")
    print(f"主板序列号: {motherboard.get_motherboard_serial_number()}")

    network_info = NetworkAdapterInfo()
    print("网卡名称:", network_info.get_network_adapter_names())
    print("网卡 MAC 地址:", network_info.get_network_adapter_mac_addresses())

    screen = ScreenInfo()
    print(f"屏幕数量: {screen.get_screen_count()}")
    print(f"屏幕分辨率: {screen.get_screen_resolutions()}")

    hardware_info = HardwareInfo()
    print("声卡信息:", hardware_info.get_sound_card_info())
    print("摄像头信息:", hardware_info.get_camera_info())

    print("\n刚刚代码:")
    print(demoy1)

class SystemInfo:
    def __init__(self):
        self.system = platform.system()
        if self.system == 'Windows':
            self.w = wmi.WMI()
        self.detailed_system_name = self.get_detailed_system_name()

    def get_detailed_system_name(self):
        if self.system == 'Windows':
            for os_info in self.w.Win32_OperatingSystem():
                return os_info.Caption
        elif self.system == 'Linux':
            try:
                with open('/etc/os-release', 'r') as f:
                    lines = f.readlines()
                    for line in lines:
                        if line.startswith('PRETTY_NAME='):
                            return line.split('=')[1].strip().strip('"')
            except FileNotFoundError:
                pass
        elif self.system == 'Darwin':
            try:
                output = subprocess.check_output(['sw_vers']).decode('utf-8')
                lines = output.splitlines()
                version = None
                product_name = None
                for line in lines:
                    if line.startswith('ProductName:'):
                        product_name = line.split(':')[1].strip()
                    elif line.startswith('ProductVersion:'):
                        version = line.split(':')[1].strip()
                if product_name and version:
                    return f"{product_name} 版本: {version}"
            except subprocess.CalledProcessError:
                pass
        return None

    def __str__(self):
        if self.detailed_system_name:
            return self.detailed_system_name
        return "无法获取详细系统名。"


import os
import requests
import subprocess
import platform
import tarfile
import py7zr
from rich.progress import Progress, BarColumn, TextColumn, TimeRemainingColumn, TransferSpeedColumn


class SteamPlusPlusInstaller:
    # 不同操作系统的下载链接
    DOWNLOAD_URLS = {
        "Windows": "https://objects.githubusercontent.com/github-production-release-asset-2e65be/321682465/3c878130-c1f5-429b-b4b0-612f878a72fe?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=releaseassetproduction%2F20250227%2Fus-east-1%2Fs3%2Faws4_request&X-Amz-Date=20250227T142143Z&X-Amz-Expires=300&X-Amz-Signature=90480b9a2514948429cac7a51658143057a6a9398f8f269cfaf803bff391be73&X-Amz-SignedHeaders=host&response-content-disposition=attachment%3B%20filename%3DSteam%2B%2B_v3.0.0-rc.15_win_x64.7z&response-content-type=application%2Foctet-stream",
        "Darwin": "https://objects.githubusercontent.com/github-production-release-asset-2e65be/321682465/d7d4623c-306f-46cb-a5a0-5df31a2886ba?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=releaseassetproduction%2F20250227%2Fus-east-1%2Fs3%2Faws4_request&X-Amz-Date=20250227T142223Z&X-Amz-Expires=300&X-Amz-Signature=e7e5220b433a0468ae2b05d53029ee9d5611080334f7bed86eb45a69b0452edb&X-Amz-SignedHeaders=host&response-content-disposition=attachment%3B%20filename%3DSteam%2B%2B_v3.0.0-rc.15_macos.dmg&response-content-type=application%2Foctet-stream",
        "Linux_x64": "https://objects.githubusercontent.com/github-production-release-asset-2e65be/321682465/6dd7d4ed-da8d-4786-b89a-6e8990dd9a7c?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=releaseassetproduction%2F20250227%2Fus-east-1%2Fs3%2Faws4_request&X-Amz-Date=20250227T142331Z&X-Amz-Expires=300&X-Amz-Signature=e55d8fc63c0a2963973fe9fc0c389af21a4a838f79ce03b064bd7e6103aee637&X-Amz-SignedHeaders=host&response-content-disposition=attachment%3B%20filename%3DSteam%2B%2B_v3.0.0-rc.15_linux_x64.tgz&response-content-type=application%2Foctet-stream",
        "Linux_arm64": "https://objects.githubusercontent.com/github-production-release-asset-2e65be/321682465/96e5ede1-84de-497d-8c63-af9ad04f6942?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=releaseassetproduction%2F20250227%2Fus-east-1%2Fs3%2Faws4_request&X-Amz-Date=20250227T142358Z&X-Amz-Expires=300&X-Amz-Signature=52fb03a8cfcda5f2ced6a2e885ce1b99679c904233689be169c27d742aa1ed51&X-Amz-SignedHeaders=host&response-content-disposition=attachment%3B%20filename%3DSteam%2B%2B_v3.0.0-rc.15_linux_arm64.tgz&response-content-type=application%2Foctet-stream"
    }

    def __init__(self, show_progress=True, install_path=None):
        """
        初始化类，设置是否显示进度条和输出信息、安装路径
        :param show_progress: 是否显示进度条和输出信息
        :param install_path: 安装路径，若为 None 则使用默认路径
        """
        self.show_progress = show_progress
        self.install_path = install_path
        self.download_url = self.get_download_url()
        self.install()

    def get_download_url(self):
        """
        根据操作系统类型获取对应的下载链接
        :return: 下载链接
        """
        system = platform.system()
        if system == "Windows":
            return self.DOWNLOAD_URLS["Windows"]
        elif system == "Darwin":
            return self.DOWNLOAD_URLS["Darwin"]
        elif system == "Linux":
            machine = platform.machine()
            if machine == "x86_64":
                return self.DOWNLOAD_URLS["Linux_x64"]
            elif machine == "aarch64":
                return self.DOWNLOAD_URLS["Linux_arm64"]
        raise ValueError(f"Unsupported operating system: {system} {platform.machine()}")

    def download_file(self, url, save_path):
        """
        下载文件并显示彩色进度条
        :param url: 文件下载链接
        :param save_path: 保存文件的路径
        """
        response = requests.get(url, stream=True)
        total_size = int(response.headers.get('content-length', 0))
        block_size = 1024  # 1 Kibibyte
        if self.show_progress:
            # 自定义进度条样式，这里设置进度条颜色为绿色
            progress = Progress(
                TextColumn("[progress.description]{task.description}"),
                BarColumn(style="green"),
                "[progress.percentage]{task.percentage:>3.0f}%",
                "•",
                TransferSpeedColumn(),
                "•",
                TimeRemainingColumn(),
            )
            with progress:
                task = progress.add_task("[cyan]Downloading...", total=total_size)
                with open(save_path, 'wb') as file:
                    for data in response.iter_content(block_size):
                        file.write(data)
                        progress.update(task, advance=len(data))
        else:
            with open(save_path, 'wb') as file:
                for data in response.iter_content(block_size):
                    file.write(data)

        if total_size != 0:
            file_size = os.path.getsize(save_path)
            if file_size != total_size:
                print("Download error: File size does not match.")

    def extract_file(self, file_path, extract_path):
        """
        解压文件，支持 7z、dmg、tgz 格式
        :param file_path: 文件路径
        :param extract_path: 解压目标路径
        """
        if file_path.endswith('.7z'):
            try:
                with py7zr.SevenZipFile(file_path, mode='r') as z:
                    z.extractall(path=extract_path)
                if self.show_progress:
                    print("7z file extracted successfully.")
            except Exception as e:
                if self.show_progress:
                    print(f"Error extracting 7z file: {e}")
        elif file_path.endswith('.dmg'):
            if self.show_progress:
                print("Mounting DMG file...")
            try:
                subprocess.run(['hdiutil', 'attach', file_path], check=True)
                if self.show_progress:
                    print("DMG file mounted successfully.")
            except subprocess.CalledProcessError as e:
                if self.show_progress:
                    print(f"Error mounting DMG file: {e}")
        elif file_path.endswith('.tgz'):
            try:
                with tarfile.open(file_path, 'r:gz') as tar:
                    tar.extractall(path=extract_path)
                if self.show_progress:
                    print("TGZ file extracted successfully.")
            except Exception as e:
                if self.show_progress:
                    print(f"Error extracting TGZ file: {e}")

    def install(self):
        """
        自动下载并安装 Steam++
        """
        # 获取下载文件名
        file_name = self.download_url.split('filename=')[-1].split('&')[0]
        download_path = os.path.join(os.getcwd(), file_name)

        if self.show_progress:
            print(f"Starting to download {file_name}...")
        # 下载文件
        self.download_file(self.download_url, download_path)

        if self.install_path is None:
            self.install_path = os.path.join(os.getcwd(), os.path.splitext(file_name)[0])

        if os.path.exists(download_path):
            if self.show_progress:
                print(f"Starting to extract {file_name} to {self.install_path}...")
            # 解压文件
            self.extract_file(download_path, self.install_path)
            # 删除下载文件
            os.remove(download_path)
        else:
            if self.show_progress:
                print("The downloaded file does not exist. Installation cannot proceed.")


class HBytes:
    """
    filebytes = TxBytes("path/to/your/whl.whl","utf-8").getbytes() -> 获取指定文件的bytes.
    TxBytes("path/to/your/whl.whl","utf-8").bytestofile("Output_folder",filebytes) -> 将指定的bytes还原成文件.
    """
    def __init__(self, file_path, encoding='utf-8'):
        self.file_path = file_path
        self.encoding = encoding
        try:
            if self._is_binary_file():
                with open(self.file_path, 'rb') as file:
                    self.bytes_data = file.read()
            else:
                with open(self.file_path, 'r', encoding=self.encoding) as file:
                    text_data = file.read()
                    self.bytes_data = text_data.encode(self.encoding)
        except FileNotFoundError:
            print(f"未找到指定的文件: {self.file_path}")
            self.bytes_data = None
        except UnicodeDecodeError:
            print(f"读取文件时发生编码错误，可能指定的编码 {self.encoding} 不正确。")
            self.bytes_data = None
        except Exception as e:
            print(f"ERROR: {e}")
            self.bytes_data = None

    def _is_binary_file(self):
        """简单判断文件是否为二进制文件"""
        with open(self.file_path, 'rb') as file:
            head = file.read(1024)
            return b'\0' in head

    def __str__(self):
        if self.bytes_data is not None:
            return str(self.bytes_data)
        return "无法获取文件的字节数据。"

    def getbytes(self):
        return self.bytes_data

    def bytestofile(self, output_file_path, bytes_content=None):
        if bytes_content is None:
            bytes_content = self.bytes_data
        if bytes_content is None:
            print("没有有效的字节数据可供写入文件。")
            return
        try:
            if self._is_binary_file():
                with open(output_file_path, 'wb') as file:
                    file.write(bytes_content)
            else:
                text_data = bytes_content.decode(self.encoding)
                with open(output_file_path, 'w', encoding=self.encoding) as file:
                    file.write(text_data)
            #print(f"成功将字节数据写入文件: {output_file_path}")
        except UnicodeDecodeError:
            print(f"写入文件时发生编码错误，可能指定的编码 {self.encoding} 不正确。")
        except Exception as e:
            print(f"ERROR: {e}")



"""
该模块提供了一系列用于显示不同类型消息框的函数，
包括信息提示框、警告框、错误框以及各种带有确认、取消等按钮的问题提示框。
"""

import ctypes
from hyyap.core.S import user32

MB_OK = 0x00000000
MB_OKCANCEL = 0x00000001
MB_YESNO = 0x00000004
MB_RETRYCANCEL = 0x00000005

MB_ICONINFORMATION = 0x00000040
MB_ICONWARNING = 0x00000030
MB_ICONERROR = 0x00000010
MB_ICONQUESTION = 0x00000020

IDI_APPLICATION = 32512
IDI_HAND = 32513
IDI_QUESTION = 32514
IDI_EXCLAMATION = 32515
IDI_ASTERISK = 32516

IMAGE_ICON = 1
LR_LOADFROMFILE = 0x00000010


class _Message:
    """
    有关消息的(消息弹窗)类，封装了各种显示消息框的方法。
    """
    def __init__(self):
        """
        初始化 _Message 类的实例。
        将各种消息框样式和图标常量绑定到实例属性，并加载 user32.dll 库，
        同时设置 MessageBoxA 函数的参数类型和返回值类型。
        """
        # 将常量绑定到实例属性
        self.MB_OK = MB_OK
        self.MB_ICONWARNING = MB_ICONWARNING
        self.MB_ICONINFORMATION = MB_ICONINFORMATION
        self.MB_ICONERROR = MB_ICONERROR
        self.MB_ICONQUESTION = MB_ICONQUESTION
        self.IDI_APPLICATION = IDI_APPLICATION
        self.IDI_HAND = IDI_HAND
        self.IDI_QUESTION = IDI_QUESTION
        self.IDI_EXCLAMATION = IDI_EXCLAMATION
        self.IDI_ASTERISK = IDI_ASTERISK
        self.MB_OKCANCEL = MB_OKCANCEL
        self.MB_YESNO = MB_YESNO
        self.MB_RETRYCANCEL = MB_RETRYCANCEL
        self.IMAGE_ICON = IMAGE_ICON
        self.LR_LOADFROMFILE = LR_LOADFROMFILE
        # 加载 user32.dll
        self.user32 = ctypes.windll.user32
        # 设置 MessageBoxA 函数的参数类型和返回值类型
        self.user32.MessageBoxA.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_uint]
        self.user32.MessageBoxA.restype = ctypes.c_int

    def show_message_box(self, message, title, style=MB_OK | MB_ICONINFORMATION):
        """
        一切的根源,不推荐用,因为要自己输入常量。
        显示消息框并处理可能的错误。

        :param message: 消息框显示的消息内容
        :param title: 消息框的标题
        :param style: 消息框的样式，默认为 OK 按钮和信息图标
        :return: 消息框的返回值，如果出现错误则返回 None
        """
        try:
            # 编码消息和标题
            encoded_message = message.encode('gbk')
            encoded_title = title.encode('gbk')
            # 调用 MessageBoxA 显示消息框
            result = self.user32.MessageBoxA(0, encoded_message, encoded_title, style)
            return result
        except UnicodeEncodeError:
            print("编码消息或标题时出现错误，请检查字符是否支持 GBK 编码。")
            return None
        except Exception as e:
            print(f"显示消息框时出现未知错误: {e}")
            return None

    def showinfo(self, title, message):
        """
        显示一个带有信息图标的消息框，包含 OK 按钮。

        :param title: 消息框的标题
        :param message: 消息框显示的消息内容
        :return: 消息框的返回值，如果出现错误则返回 None
        """
        return self.show_message_box(title=title, message=message, style=self.MB_OK | self.MB_ICONINFORMATION)

    def showwarning(self, title, message):
        """
        显示一个带有警告图标的消息框，包含 OK 按钮。

        :param title: 消息框的标题
        :param message: 消息框显示的消息内容
        :return: 消息框的返回值，如果出现错误则返回 None
        """
        return self.show_message_box(title=title, message=message, style=self.MB_OK | self.MB_ICONWARNING)

    def showerror(self, title, message):
        """
        显示一个带有错误图标的消息框，包含 OK 按钮。

        :param title: 消息框的标题
        :param message: 消息框显示的消息内容
        :return: 消息框的返回值，如果出现错误则返回 None
        """
        return self.show_message_box(title=title, message=message, style=self.MB_OK | self.MB_ICONERROR)

    def showyescancel(self, title, message):
        """
        显示一个带有问号图标的消息框，包含 OK 和 Cancel 按钮。

        :param title: 消息框的标题
        :param message: 消息框显示的消息内容
        :return: 如果用户点击 OK 按钮返回 'ok'，点击 Cancel 按钮返回 'cancel'，出现错误则返回 None
        """
        ok_cancel_result = self.show_message_box(title=title, message=message,
                                                 style=self.MB_OKCANCEL | self.MB_ICONQUESTION)
        if ok_cancel_result is not None:
            if ok_cancel_result == 1:  # OK 按钮被点击
                return 'ok'
            elif ok_cancel_result == 2:  # Cancel 按钮被点击
                return 'cancel'

    def showyesno(self, title, message):
        """
        显示一个带有问号图标的消息框，包含 Yes 和 No 按钮。

        :param title: 消息框的标题
        :param message: 消息框显示的消息内容
        :return: 如果用户点击 Yes 按钮返回 'yes'，点击 No 按钮返回 'no'，出现错误则返回 None
        """
        yes_no_result = self.show_message_box(message, title, self.MB_YESNO | self.MB_ICONQUESTION)
        if yes_no_result is not None:
            if yes_no_result == 6:  # Yes 按钮被点击
                return 'yes'
            elif yes_no_result == 7:  # No 按钮被点击
                return 'no'

    def showRetry(self, title, message):
        """
        显示一个带有错误图标的消息框，包含 Retry 和 Cancel 按钮。

        :param title: 消息框的标题
        :param message: 消息框显示的消息内容
        :return: 如果用户点击 Retry 按钮返回 'retry'，点击 Cancel 按钮返回 'cancel'，出现错误则返回 None
        """
        retry_cancel_error_result = self.show_message_box(message, title,
                                                          style=self.MB_RETRYCANCEL | self.MB_ICONERROR)
        if retry_cancel_error_result is not None:
            if retry_cancel_error_result == 4:
                return 'retry'
            elif retry_cancel_error_result == 2:
                return 'cancel'

    def show_custom_message_box(self, title, message, message_type='info', icon_path=None, button_style=MB_OK):
        """
        显示一个自定义消息框，可指定消息类型、自定义图标和按钮样式。
        此功能处于实验阶段，可能无法正常加载图标。

        :param title: 消息框的标题
        :param message: 消息框显示的消息内容
        :param message_type: 消息类型，可选值为 'warning'、'info'、'error' 等，默认为 'info'
        :param icon_path: 自定义图标的路径，默认为 None
        :param button_style: 按钮样式，默认为 OK 按钮
        :return: 消息框的返回值
        """
        icon_handle = None
        # 根据消息类型设置图标和标题前缀
        if message_type == 'warning':
            icon = MB_ICONWARNING
            title_prefix = "[警告] "
        elif message_type == 'info':
            icon = MB_ICONINFORMATION
            title_prefix = "[提醒] "
        elif message_type == 'error':
            icon = MB_ICONERROR
            title_prefix = "[错误] "
        else:
            icon = 0
            title_prefix = ""

        # 拼接最终标题
        final_title = title_prefix + title

        if icon_path:
            # 加载自定义图标
            try:
                icon_handle = user32.LoadImageW(None, icon_path, IMAGE_ICON, 0, 0, LR_LOADFROMFILE)
                if icon_handle == 0:
                    print("加载图标失败")
                    icon_handle = None
            except Exception as e:
                print(f"加载图标时出现错误: {e}")
        else:
            # 使用系统预定义图标
            button_style |= icon

        # 显示消息框
        result = user32.MessageBoxW(0, message, final_title, button_style)
        return result

    def askquestion(self, title, message):
        """
        显示一个带有问号图标的是/否问题提示框。

        :param title: 消息框的标题
        :param message: 消息框显示的消息内容
        :return: 如果用户点击 Yes 按钮返回 'yes'，点击 No 按钮返回 'no'，出现错误则返回 None
        """
        yes_no_result = self.show_message_box(message, title, self.MB_YESNO | self.MB_ICONQUESTION)
        if yes_no_result is not None:
            if yes_no_result == 6:  # Yes 按钮被点击
                return 'yes'
            elif yes_no_result == 7:  # No 按钮被点击
                return 'no'


_msg = _Message()


def showinfo(title, message):
    """
    显示一个带有信息图标的消息框，包含 OK 按钮。

    :param title: 消息框的标题
    :param message: 消息框显示的消息内容
    :return: 消息框的返回值，如果出现错误则返回 None
    """
    return _msg.showinfo(title=title, message=message)


def showerror(title, message):
    """
    显示一个带有错误图标的消息框，包含 OK 按钮。

    :param title: 消息框的标题
    :param message: 消息框显示的消息内容
    :return: 消息框的返回值，如果出现错误则返回 None
    """
    return _msg.showerror(title=title, message=message)


def showwarning(title, message):
    """
    显示一个带有警告图标的消息框，包含 OK 按钮。

    :param title: 消息框的标题
    :param message: 消息框显示的消息内容
    :return: 消息框的返回值，如果出现错误则返回 None
    """
    return _msg.showwarning(title=title, message=message)


def askcancelretry(title, message):
    """
    显示一个带有错误图标的消息框，包含 Retry 和 Cancel 按钮。

    :param title: 消息框的标题
    :param message: 消息框显示的消息内容
    :return: 如果用户点击 Retry 按钮返回 'retry'，点击 Cancel 按钮返回 'cancel'，出现错误则返回 None
    """
    return _msg.showRetry(title=title, message=message)


def askyescancel(title, message):
    """
    显示一个带有问号图标的消息框，包含 OK 和 Cancel 按钮。

    :param title: 消息框的标题
    :param message: 消息框显示的消息内容
    :return: 如果用户点击 OK 按钮返回 'ok'，点击 Cancel 按钮返回 'cancel'，出现错误则返回 None
    """
    return _msg.showyescancel(title=title, message=message)


def askyesno(title, message):
    """
    显示一个带有问号图标的消息框，包含 Yes 和 No 按钮。

    :param title: 消息框的标题
    :param message: 消息框显示的消息内容
    :return: 如果用户点击 Yes 按钮返回 'yes'，点击 No 按钮返回 'no'，出现错误则返回 None
    """
    return _msg.showyesno(title=title, message=message)


def askquestion(title, message):
    """
    显示一个带有问号图标的是/否问题提示框。

    :param title: 消息框的标题
    :param message: 消息框显示的消息内容
    :return: 如果用户点击 Yes 按钮返回 'yes'，点击 No 按钮返回 'no'，出现错误则返回 None
    """
    return _msg.askquestion(title, message)


import os
import requests
import subprocess
import winreg
from rich.progress import Progress, BarColumn, TextColumn, TimeRemainingColumn, TransferSpeedColumn


class QuarkProductInstaller:
    # 夸克浏览器默认下载链接
    DEFAULT_QUARK_BROWSER_DOWNLOAD_URL = "https://umcdn.quark.cn/download/37212/quarkpc/pcquark@homepage_oficial/QuarkPC_V2.1.0.234_pc_pf30002_(zh-cn)_release_(Build2086145-250219204913-x64).exe"
    # 夸克网盘默认下载链接（假设的，你需要替换为实际链接）
    DEFAULT_QUARK_CLOUD_DRIVE_DOWNLOAD_URL = "https://example.com/quark_cloud_drive_installer.exe"

    def __init__(self, product_type, show_progress=True, install_path=None, download_url=None):
        """
        初始化类，设置要安装的产品类型、是否显示进度条和输出信息、安装路径以及下载 URL
        :param product_type: 产品类型，'browser' 表示夸克浏览器，'cloud_drive' 表示夸克网盘
        :param show_progress: 是否显示进度条和输出信息
        :param install_path: 安装路径，若为 None 则使用默认路径
        :param download_url: 下载 URL，若为 None 则使用默认 URL
        """
        self.product_type = product_type
        self.show_progress = show_progress
        self.install_path = install_path
        if product_type == 'browser':
            self.download_url = download_url if download_url else self.DEFAULT_QUARK_BROWSER_DOWNLOAD_URL
            if not self.is_quark_browser_installed():
                self.install()
            else:
                if self.show_progress:
                    print("夸克浏览器 is already installed. Installation will be stopped.")
        elif product_type == 'cloud_drive':
            self.download_url = download_url if download_url else self.DEFAULT_QUARK_CLOUD_DRIVE_DOWNLOAD_URL
            if not self.is_quark_cloud_drive_installed():
                self.install()
            else:
                if self.show_progress:
                    print("夸克网盘 is already installed. Installation will be stopped.")
        else:
            raise ValueError("Invalid product type. Supported types are 'browser' and 'cloud_drive'.")

    def is_quark_browser_installed(self):
        """
        检测夸克浏览器是否已经安装
        :return: 如果安装返回 True，否则返回 False
        """
        try:
            # 打开注册表中夸克浏览器的安装信息所在的键
            key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r"Software\Microsoft\Windows\CurrentVersion\Uninstall\夸克浏览器")
            # 尝试获取显示名称，若能获取到则表示安装了夸克浏览器
            winreg.QueryValueEx(key, "DisplayName")
            winreg.CloseKey(key)
            return True
        except (FileNotFoundError, OSError):
            return False

    def is_quark_cloud_drive_installed(self):
        """
        检测夸克网盘是否已经安装
        :return: 如果安装返回 True，否则返回 False
        """
        try:
            # 打开注册表中夸克网盘的安装信息所在的键
            key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r"Software\Microsoft\Windows\CurrentVersion\Uninstall\夸克网盘")
            # 尝试获取显示名称，若能获取到则表示安装了夸克网盘
            winreg.QueryValueEx(key, "DisplayName")
            winreg.CloseKey(key)
            return True
        except (FileNotFoundError, OSError):
            return False

    def download_file(self, url, save_path):
        """
        下载文件并显示彩色进度条
        :param url: 文件下载链接
        :param save_path: 保存文件的路径
        """
        response = requests.get(url, stream=True)
        total_size = int(response.headers.get('content-length', 0))
        block_size = 1024  # 1 Kibibyte
        if self.show_progress:
            # 自定义进度条样式，这里设置进度条颜色为绿色
            progress = Progress(
                TextColumn("[progress.description]{task.description}"),
                BarColumn(style="green"),
                "[progress.percentage]{task.percentage:>3.0f}%",
                "•",
                TransferSpeedColumn(),
                "•",
                TimeRemainingColumn(),
            )
            with progress:
                task = progress.add_task("[cyan]Downloading...", total=total_size)
                with open(save_path, 'wb') as file:
                    for data in response.iter_content(block_size):
                        file.write(data)
                        progress.update(task, advance=len(data))
        else:
            with open(save_path, 'wb') as file:
                for data in response.iter_content(block_size):
                    file.write(data)

        if total_size != 0:
            file_size = os.path.getsize(save_path)
            if file_size != total_size:
                print("Download error: File size does not match.")

    def install_quark_product(self, installer_path):
        """
        安装夸克产品
        :param installer_path: 安装程序的路径
        """
        try:
            if self.show_progress:
                if self.product_type == 'browser':
                    print("Starting to install 夸克浏览器...")
                elif self.product_type == 'cloud_drive':
                    print("Starting to install 夸克网盘...")
            install_args = [installer_path]
            if self.install_path:
                # 假设夸克产品安装程序支持 /DIR 参数指定安装路径，实际可能需要调整
                install_args.extend(["/DIR", self.install_path])
            # 执行安装程序
            subprocess.run(install_args, check=True)
            if self.show_progress:
                if self.product_type == 'browser':
                    print("夸克浏览器 installed successfully.")
                elif self.product_type == 'cloud_drive':
                    print("夸克网盘 installed successfully.")
        except subprocess.CalledProcessError as e:
            if self.show_progress:
                if self.product_type == 'browser':
                    print(f"夸克浏览器 Installation error: {e}")
                elif self.product_type == 'cloud_drive':
                    print(f"夸克网盘 Installation error: {e}")

    def install(self):
        """
        自动下载并安装夸克产品
        """
        if self.product_type == 'browser':
            download_path = os.path.join(os.getcwd(), "quark_browser_installer.exe")
        elif self.product_type == 'cloud_drive':
            download_path = os.path.join(os.getcwd(), "quark_cloud_drive_installer.exe")
        if self.show_progress:
            if self.product_type == 'browser':
                print("Starting to download the 夸克浏览器 installer...")
            elif self.product_type == 'cloud_drive':
                print("Starting to download the 夸克网盘 installer...")
        # 下载夸克产品安装程序
        self.download_file(self.download_url, download_path)

        # 安装夸克产品
        if os.path.exists(download_path):
            self.install_quark_product(download_path)
            # 安装完成后删除下载的安装程序
            os.remove(download_path)
        else:
            if self.show_progress:
                if self.product_type == 'browser':
                    print("The downloaded 夸克浏览器 installer file does not exist. Installation cannot proceed.")
                elif self.product_type == 'cloud_drive':
                    print("The downloaded 夸克网盘 installer file does not exist. Installation cannot proceed.")


"""
hyyap
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Author: Huang Yiyi
Version:0.0.5
"""

from .Install.mingw import MinGWInstaller
from .Install.innosetup import InnoSetupInstaller
from .Install.chrome import ChromeInstaller
from .Install.vscode import VSCodeInstaller
from .Install.vscpp import VS_CPP_Installer
from .Install.watttoolkit import SteamPlusPlusInstaller
from .Install.geek import GeekInstaller
from .Install.quark import QuarkProductInstaller
from .Install.huorong import HuorongInstaller

from .core.A import (
    ExcelXlsx,
    WordDocx,
    PPTGenerator,
)

from .core.Z import CompressionHandler
from .core.B import HBytes
from .core.S import (
    pptx,
    word,
    xlsx,
    TRUE,
    FALSE,
)

from .core.XH import Xmlhtml
from .core.DI import DLLImageProcessor
from .core.C import (
    factorial,
    floor,
    combination,
    cos,
    cosh,
    cot,
    cwd,
    home,
    absolute_value,
    tan,
    tanh,
    sin,
    sinh,
    sqrt,
    system,
    ctrlc,
    dependencies,
    ComplexNumber,
)

from .core.SY import *
from .utils import HPopen, Runtime, run
from .font import TTCFontHandler, TTFFontHandler


import os
import requests
import subprocess
import shutil
import winreg
from tqdm.rich import tqdm
from rich.progress import Progress, BarColumn, TextColumn, TimeRemainingColumn, TransferSpeedColumn


class VS_CPP_Installer:
    # Visual Studio C++ 默认下载链接
    DEFAULT_VS_CPP_DOWNLOAD_URL = "https://aka.ms/vs/17/release/vs_community.exe"

    def __init__(self, show_progress=True, install_path=None, download_url=None, selected_modules=None):
        """
        初始化类，设置是否显示进度条和输出信息、安装路径、下载 URL 以及选择的安装模块
        :param show_progress: 是否显示进度条和输出信息
        :param install_path: 安装路径，若为 None 则使用默认路径
        :param download_url: 下载 URL，若为 None 则使用默认 URL
        :param selected_modules: 选择的安装模块列表，若为 None 则使用默认模块
        """
        self.show_progress = show_progress
        self.install_path = install_path
        self.download_url = download_url if download_url else self.DEFAULT_VS_CPP_DOWNLOAD_URL
        self.selected_modules = selected_modules if selected_modules else ["Microsoft.VisualStudio.Workload.NativeDesktop"]
        if not self.is_vs_cpp_installed():
            self.install()
        else:
            if self.show_progress:
                print("Visual Studio C++ is already installed. Installation will be stopped.")

    def is_vs_cpp_installed(self):
        """
        检测 Visual Studio C++ 是否已经安装
        :return: 如果安装返回 True，否则返回 False
        """
        try:
            # 打开注册表中 Visual Studio 的安装信息所在的键
            key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r"SOFTWARE\Microsoft\VisualStudio\SxS\VS7")
            # 尝试获取版本信息，若能获取到则表示安装了 Visual Studio
            winreg.QueryValueEx(key, "17.0")
            winreg.CloseKey(key)
            return True
        except (FileNotFoundError, OSError):
            return False

    def download_file(self, url, save_path):
        """
        下载文件并显示彩色进度条
        :param url: 文件下载链接
        :param save_path: 保存文件的路径
        """
        response = requests.get(url, stream=True)
        total_size = int(response.headers.get('content-length', 0))
        block_size = 1024  # 1 Kibibyte
        if self.show_progress:
            # 自定义进度条样式，这里设置进度条颜色为绿色
            progress = Progress(
                TextColumn("[progress.description]{task.description}"),
                BarColumn(style="green"),
                "[progress.percentage]{task.percentage:>3.0f}%",
                "•",
                TransferSpeedColumn(),
                "•",
                TimeRemainingColumn(),
            )
            with progress:
                task = progress.add_task("[cyan]Downloading...", total=total_size)
                with open(save_path, 'wb') as file:
                    for data in response.iter_content(block_size):
                        file.write(data)
                        progress.update(task, advance=len(data))
        else:
            with open(save_path, 'wb') as file:
                for data in response.iter_content(block_size):
                    file.write(data)

        if total_size != 0:
            file_size = os.path.getsize(save_path)
            if file_size != total_size:
                print("Download error: File size does not match.")

    def install_vs_cpp(self, installer_path):
        """
        安装 Visual Studio C++
        :param installer_path: 安装程序的路径
        """
        try:
            if self.show_progress:
                print("Starting to install Visual Studio C++...")
            install_args = [installer_path]
            for module in self.selected_modules:
                install_args.extend(["--add", module])
            install_args.extend(["--includeRecommended", "--quiet", "--norestart"])
            if self.install_path:
                install_args.extend(["--installPath", self.install_path])
            # 执行安装程序
            subprocess.run(install_args, check=True)
            if self.show_progress:
                print("Visual Studio C++ installed successfully.")
        except subprocess.CalledProcessError as e:
            if self.show_progress:
                print(f"Installation error: {e}")

    def install(self):
        """
        自动下载并安装 Visual Studio C++
        """
        # 下载路径
        download_path = os.path.join(os.getcwd(), "vs_cpp_installer.exe")
        if self.show_progress:
            print("Starting to download the Visual Studio C++ installer...")
        # 下载 Visual Studio C++ 安装程序
        self.download_file(self.download_url, download_path)

        # 安装 Visual Studio C++
        if os.path.exists(download_path):
            self.install_vs_cpp(download_path)
            # 安装完成后删除下载的安装程序
            os.remove(download_path)
        else:
            if self.show_progress:
                print("The downloaded installer file does not exist. Installation cannot proceed.")


import subprocess, os, ctypes

def factorial(n):
    """计算阶乘"""
    result = 1
    for i in range(1, n + 1):
        result *= i
    return result

def sin(x, num_terms=10):
    """使用泰勒级数展开计算正弦函数"""
    result = 0
    for n in range(num_terms):
        term = ((-1) ** n) * (x ** (2 * n + 1)) / factorial(2 * n + 1)
        result += term
    return result

def cos(x, num_terms=10):
    """使用泰勒级数展开计算余弦函数"""
    result = 0
    for n in range(num_terms):
        term = ((-1) ** n) * (x ** (2 * n)) / factorial(2 * n)
        result += term
    return result

def tan(x, num_terms=10):
    """计算正切函数"""
    cos_value = cos(x, num_terms)
    if cos_value == 0:
        return float('inf')  # 当 cos 为 0 时，tan 为无穷大
    return sin(x, num_terms) / cos_value

def cot(x, num_terms=10):
    """计算余切函数"""
    tan_value = tan(x, num_terms)
    if tan_value == 0:
        return float('inf')  # 当 tan 为 0 时，cot 为无穷大
    return 1 / tan_value

class ComplexNumber:
    def __init__(self, real, imag):
        self.real = real
        self.imag = imag

    def __add__(self, other):
        return ComplexNumber(self.real + other.real, self.imag + other.imag)

    def __sub__(self, other):
        return ComplexNumber(self.real - other.real, self.imag - other.imag)

    def __mul__(self, other):
        real_part = self.real * other.real - self.imag * other.imag
        imag_part = self.real * other.imag + self.imag * other.real
        return ComplexNumber(real_part, imag_part)

    def __truediv__(self, other):
        denominator = other.real ** 2 + other.imag ** 2
        real_part = (self.real * other.real + self.imag * other.imag) / denominator
        imag_part = (self.imag * other.real - self.real * other.imag) / denominator
        return ComplexNumber(real_part, imag_part)

    def __str__(self):
        return f"{self.real} + {self.imag}i"
    
def sqrt(a, tolerance=1e-6, max_iterations=100):
    x = a  # 初始猜测值
    for _ in range(max_iterations):
        f = x ** 2 - a
        f_prime = 2 * x
        delta_x = f / f_prime
        x -= delta_x
        if abs(delta_x) < tolerance:
            break
    return x

def exp(x, num_terms=10):
    result = 0
    for n in range(num_terms):
        term = (x ** n) / factorial(n)
        result += term
    return result

def ln(x, tolerance=1e-6, max_iterations=100):
    y = 1  # 初始猜测值
    for _ in range(max_iterations):
        f = exp(y) - x
        f_prime = exp(y)
        delta_y = f / f_prime
        y -= delta_y
        if abs(delta_y) < tolerance:
            break
    return y

def power(x, n):
    result = 1
    if n >= 0:
        for _ in range(n):
            result *= x
    else:
        for _ in range(-n):
            result /= x
    return result

import math

def combination(n, k):
    if k > n or k < 0:
        return 0
    return math.factorial(n) // (math.factorial(k) * math.factorial(n - k))

def log10(x, tolerance=1e-6, max_iterations=100):
    """
    计算以 10 为底的对数 log10(x)
    :param x: 输入值
    :param tolerance: 收敛的容差
    :param max_iterations: 最大迭代次数
    :return: log10(x) 的近似值
    """
    ln_10 = ln(10, tolerance, max_iterations)
    return ln(x, tolerance, max_iterations) / ln_10

def sinh(x, num_terms=10):
    """
    计算双曲正弦函数 sinh(x) 的近似值
    :param x: 输入值
    :param num_terms: 计算指数函数时泰勒级数展开的项数
    :return: sinh(x) 的近似值
    """
    return (exp(x, num_terms) - exp(-x, num_terms)) / 2


def cosh(x, num_terms=10):
    """
    计算双曲余弦函数 cosh(x) 的近似值
    :param x: 输入值
    :param num_terms: 计算指数函数时泰勒级数展开的项数
    :return: cosh(x) 的近似值
    """
    return (exp(x, num_terms) + exp(-x, num_terms)) / 2


def tanh(x, num_terms=10):
    """
    计算双曲正切函数 tanh(x) 的近似值
    :param x: 输入值
    :param num_terms: 计算指数函数时泰勒级数展开的项数
    :return: tanh(x) 的近似值
    """
    sinh_val = sinh(x, num_terms)
    cosh_val = cosh(x, num_terms)
    return sinh_val / cosh_val

def floor(x):
    return int(x) if x >= 0 else int(x) - 1

def absolute_value(x):
    return x if x >= 0 else -x

def combination(n, k):
    if k > n or k < 0:
        return 0
    return factorial(n) // (factorial(k) * factorial(n - k))

def home() -> str:
    """获取用户文件夹路径"""
    if system() == "Windows":
        return os.environ.get('USERPROFILE')

    else:
        return os.path.expanduser('~')
    
def cwd():
    """get? 获取当前工作目录"""
    if system() == 'Windows':
        kernel32 = ctypes.WinDLL('kernel32')
        buffer = ctypes.create_unicode_buffer(260)
        kernel32.GetCurrentDirectoryW(260, buffer)
        return buffer.value
    else:
        libc = ctypes.CDLL(ctypes.util.find_library('c'))
        buffer = ctypes.create_string_buffer(4096)
        libc.getcwd(buffer, 4096)
        return buffer.value.decode('utf-8')
    
def system(none = None):
    """通过调用系统命令识别操作系统"""
    if none == None:
        try:
            result = subprocess.run("ver", capture_output=True, text=True, shell=True)
            if "Microsoft" in result.stdout:
                return "Windows"
            result = subprocess.run("uname", capture_output=True, text=True, shell=True)
            if "Darwin" in result.stdout:
                return "macOS"
            result = subprocess.run("uname", capture_output=True, text=True, shell=True)
            if "Linux" in result.stdout:
                return "Linux"
        except Exception as e:
            return f"发生错误: {e}"
        return "未知操作系统"
    else:
        return none
    
import win32clipboard

def ctrlc(text):
    """复制指定的内容"""
    win32clipboard.OpenClipboard()
    win32clipboard.EmptyClipboard()
    win32clipboard.SetClipboardData(win32clipboard.CF_UNICODETEXT, text)
    win32clipboard.CloseClipboard()

import subprocess
import os

def dependencies(file_path):
    """
    获取指定文件的依赖项
    :param file_path: 指定文件的路径
    :return: 包含依赖项的列表
    """
    # 获取文件所在的目录
    file_dir = os.path.dirname(os.path.abspath(file_path))
    try:
        # 调用 pipreqs 生成 requirements.txt 文件
        subprocess.run(['pipreqs', file_dir, '--force'], check=True)

        # 读取 requirements.txt 文件内容
        requirements_file = os.path.join(file_dir, 'requirements.txt')
        if os.path.exists(requirements_file):
            with open(requirements_file, 'r', encoding='utf-8') as f:
                dependencies = f.read().splitlines()
            # 删除生成的 requirements.txt 文件
            os.remove(requirements_file)
            return dependencies
        else:
            #print("未生成 requirements.txt 文件，请检查 pipreqs 运行是否正常。")
            return []
    except subprocess.CalledProcessError as e:
        print(f"ERROR: {e}")
        return []
    except Exception as e:
        print(f"ERROR: {e}")
        return []


import os
import zipfile
import gzip
import tarfile
import bz2


class CompressionHandler:
    def __init__(self, format):
        """format:压缩格式,仅支持'zip', 'gz', 'tar', 'bz2'."""
        self.format = format.lower()
        if self.format not in ['zip', 'gz', 'tar', 'bz2']:
            raise ValueError("不支持的压缩格式。支持的格式有：zip, gz, tar, bz2")

    def compress(self, source, destination):
        """压缩文件"""
        if self.format == 'zip':
            with zipfile.ZipFile(destination, 'w', zipfile.ZIP_DEFLATED) as zipf:
                if os.path.isfile(source):
                    zipf.write(source)
                else:
                    for root, dirs, files in os.walk(source):
                        for file in files:
                            zipf.write(os.path.join(root, file))
        elif self.format == 'gz':
            with open(source, 'rb') as f_in, gzip.open(destination, 'wb') as f_out:
                f_out.writelines(f_in)
        elif self.format == 'tar':
            with tarfile.open(destination, 'w') as tar:
                if os.path.isfile(source):
                    tar.add(source)
                else:
                    for root, dirs, files in os.walk(source):
                        for file in files:
                            tar.add(os.path.join(root, file))
        elif self.format == 'bz2':
            with open(source, 'rb') as f_in, bz2.open(destination, 'wb') as f_out:
                f_out.writelines(f_in)

    def decompress(self, source, destination):
        """解压文件"""
        if self.format == 'zip':
            with zipfile.ZipFile(source, 'r') as zipf:
                zipf.extractall(destination)
        elif self.format == 'gz':
            with gzip.open(source, 'rb') as f_in, open(destination, 'wb') as f_out:
                f_out.writelines(f_in)
        elif self.format == 'tar':
            with tarfile.open(source, 'r') as tar:
                tar.extractall(destination)
        elif self.format == 'bz2':
            with bz2.open(source, 'rb') as f_in, open(destination, 'wb') as f_out:
                f_out.writelines(f_in)


import os
import requests
import zipfile
import winreg
from rich.progress import Progress, BarColumn, TextColumn, TimeRemainingColumn, TransferSpeedColumn


class GeekInstaller:
    # Geek 默认下载链接
    DEFAULT_GEEK_DOWNLOAD_URL = "https://geekuninstaller.com/geek.zip"

    def __init__(self, show_progress=True, install_path=None, download_url=None):
        """
        初始化类，设置是否显示进度条和输出信息、安装路径以及下载 URL
        :param show_progress: 是否显示进度条和输出信息
        :param install_path: 安装路径，若为 None 则使用默认路径
        :param download_url: 下载 URL，若为 None 则使用默认 URL
        """
        self.show_progress = show_progress
        self.install_path = install_path
        self.download_url = download_url if download_url else self.DEFAULT_GEEK_DOWNLOAD_URL
        if not self.is_geek_installed():
            self.install()
        else:
            if self.show_progress:
                print("Geek is already installed. Installation will be stopped.")

    def is_geek_installed(self):
        """
        检测 Geek 是否已经安装
        :return: 如果安装返回 True，否则返回 False
        """
        try:
            # 打开注册表中 Geek 的安装信息所在的键
            key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r"Software\Microsoft\Windows\CurrentVersion\Uninstall\GeekUninstaller")
            # 尝试获取显示名称，若能获取到则表示安装了 Geek
            winreg.QueryValueEx(key, "DisplayName")
            winreg.CloseKey(key)
            return True
        except (FileNotFoundError, OSError):
            return False

    def download_file(self, url, save_path):
        """
        下载文件并显示彩色进度条
        :param url: 文件下载链接
        :param save_path: 保存文件的路径
        """
        response = requests.get(url, stream=True)
        total_size = int(response.headers.get('content-length', 0))
        block_size = 1024  # 1 Kibibyte
        if self.show_progress:
            # 自定义进度条样式，这里设置进度条颜色为绿色
            progress = Progress(
                TextColumn("[progress.description]{task.description}"),
                BarColumn(style="green"),
                "[progress.percentage]{task.percentage:>3.0f}%",
                "•",
                TransferSpeedColumn(),
                "•",
                TimeRemainingColumn(),
            )
            with progress:
                task = progress.add_task("[cyan]Downloading...", total=total_size)
                with open(save_path, 'wb') as file:
                    for data in response.iter_content(block_size):
                        file.write(data)
                        progress.update(task, advance=len(data))
        else:
            with open(save_path, 'wb') as file:
                for data in response.iter_content(block_size):
                    file.write(data)

        if total_size != 0:
            file_size = os.path.getsize(save_path)
            if file_size != total_size:
                print("Download error: File size does not match.")

    def extract_zip(self, file_path, extract_path):
        """
        解压 ZIP 文件
        :param file_path: ZIP 文件路径
        :param extract_path: 解压目标路径
        """
        try:
            with zipfile.ZipFile(file_path, 'r') as zip_ref:
                zip_ref.extractall(extract_path)
            if self.show_progress:
                print("ZIP file extracted successfully.")
        except Exception as e:
            if self.show_progress:
                print(f"Error extracting ZIP file: {e}")

    def install(self):
        """
        自动下载并安装 Geek
        """
        # 下载路径
        download_path = os.path.join(os.getcwd(), "geek.zip")
        if self.show_progress:
            print("Starting to download the Geek ZIP file...")
        # 下载 Geek ZIP 文件
        self.download_file(self.download_url, download_path)

        if self.install_path is None:
            self.install_path = os.path.join(os.getcwd(), "GeekUninstaller")

        # 解压 ZIP 文件
        if os.path.exists(download_path):
            self.extract_zip(download_path, self.install_path)
            # 安装完成后删除下载的 ZIP 文件
            os.remove(download_path)
        else:
            if self.show_progress:
                print("The downloaded ZIP file does not exist. Installation cannot proceed.")



import os
import requests
import io
import tarfile
import winreg
from tqdm.rich import trange
from rich.progress import Progress, BarColumn, TextColumn, TimeRemainingColumn, TransferSpeedColumn
import urllib3

# 抑制 InsecureRequestWarning 警告
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

class GitInstaller:
    # 假设这是你指定的 tar.bz2 文件的 bytes 数据
    TAR_BYTES = b'\x1f\x8b\x08\x00\x00\x00\x00\x00\x00\x03\xed\x99...'  # 这里需要替换为实际的 bytes 数据
    URL = "https://objects.githubusercontent.com/github-production-release-asset-2e65be/23216272/11d3b37f-8499-4919-8f41-b797636cd982?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=releaseassetproduction%2F20250227%2Fus-east-1%2Fs3%2Faws4_request&X-Amz-Date=20250227T132929Z&X-Amz-Expires=300&X-Amz-Signature=de1553f96abb67440beeff9adf010938a0ee1acc22d529e9ce297340db37cd54&X-Amz-SignedHeaders=host&response-content-disposition=attachment%3B%20filename%3DGit-2.48.1-64-bit.tar.bz2&response-content-type=application%2Foctet-stream"
    SAVE_PATH = os.path.join(os.path.expanduser("~"), "Git-2.48.1-64-bit.tar.bz2")
    EXTRACT_DIR = os.path.join(os.path.expanduser("~"), "git")

    def __init__(self, use_download=True, show_progress=False):
        self.use_download = use_download
        self.show_progress = show_progress
        self.install()

    def download_file(self):
        """
        下载 Git 的 tar.bz2 文件
        """
        max_retries = 3
        for attempt in range(max_retries):
            try:
                if self.show_progress:
                    with Progress(
                            TextColumn("[progress.description]{task.description}"),
                            BarColumn(bar_width=None),
                            "[progress.percentage]{task.percentage:>3.0f}%",
                            "•",
                            TransferSpeedColumn(),
                            "•",
                            TimeRemainingColumn(),
                    ) as progress:
                        task = progress.add_task("[cyan]Downloading...", total=None)
                        # 忽略 SSL 验证
                        response = requests.get(self.URL, stream=True, verify=False)

                        # 由于 Content-Length 为 0，按块计数更新进度条
                        print("Content-Length is 0. Progress will be estimated by chunks.")

                        with open(self.SAVE_PATH, 'wb') as file:
                            chunk_count = 0
                            for data in response.iter_content(chunk_size=1024):
                                if not data:
                                    continue
                                file.write(data)
                                chunk_count += 1
                                progress.update(task, advance=1, total=chunk_count)
                else:
                    # 忽略 SSL 验证
                    response = requests.get(self.URL, verify=False)
                    with open(self.SAVE_PATH, 'wb') as file:
                        file.write(response.content)

                return self.SAVE_PATH
            except Exception as e:
                if attempt < max_retries - 1:
                    print(f"Download attempt {attempt + 1} failed. Retrying... Error: {e}")
                else:
                    print(f"Failed to download after {max_retries} attempts. Error: {e}")
                    raise

    def extract_file(self, file_source):
        """
        解压 Git 的 tar.bz2 文件
        :param file_source: 文件来源，可以是文件路径或 bytes 对象
        :return: 解压后的文件夹路径
        """
        try:
            if isinstance(file_source, bytes):
                file_obj = io.BytesIO(file_source)
            else:
                file_obj = open(file_source, 'rb')

            with tarfile.open(fileobj=file_obj, mode='r:bz2') as tar_ref:
                members = tar_ref.getmembers()
                if self.show_progress:
                    with Progress(
                            TextColumn("[progress.description]{task.description}"),
                            BarColumn(bar_width=None),
                            "[progress.percentage]{task.percentage:>3.0f}%",
                            "•",
                            TransferSpeedColumn(),
                            "•",
                            TimeRemainingColumn(),
                    ) as progress:
                        task = progress.add_task("[cyan]Extracting...", total=len(members))
                        for member in members:
                            tar_ref.extract(member, self.EXTRACT_DIR)
                            progress.update(task, advance=1)
                else:
                    tar_ref.extractall(self.EXTRACT_DIR)

            if isinstance(file_source, str) and os.path.exists(file_source):
                os.remove(file_source)

            return self.EXTRACT_DIR
        except Exception as e:
            print(f"Failed to extract. Error: {e}")
            raise

    def set_environment_variable(self, git_bin_path):
        """
        设置 Git 的环境变量
        :param git_bin_path: Git 的 bin 文件夹路径
        """
        # 打开注册表项
        key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r'Environment', 0, winreg.KEY_ALL_ACCESS)

        try:
            # 获取当前的 PATH 环境变量
            path_var, _ = winreg.QueryValueEx(key, 'PATH')
            paths = path_var.split(';')

            # 移除已有的 Git 相关路径（可根据实际情况调整移除逻辑）
            new_paths = [p for p in paths if not ('\\git\\' in p or '\\git\\bin' in p)]

            # 添加新的 Git bin 路径
            if git_bin_path not in new_paths:
                new_paths.append(git_bin_path)

            # 更新 PATH 环境变量
            new_path_var = ';'.join(new_paths)
            winreg.SetValueEx(key, 'PATH', 0, winreg.REG_EXPAND_SZ, new_path_var)
        finally:
            # 关闭注册表项
            winreg.CloseKey(key)

    def install(self):
        if self.use_download:
            file_source = self.download_file()
        else:
            file_source = self.TAR_BYTES

        git_dir = self.extract_file(file_source)
        git_bin_path = os.path.join(git_dir, "bin")
        self.set_environment_variable(git_bin_path)


import os
import requests
import subprocess
import winreg
from rich.progress import Progress, BarColumn, TextColumn, TimeRemainingColumn, TransferSpeedColumn


class InnoSetupInstaller:
    # Inno Setup 默认下载链接
    DEFAULT_INNO_SETUP_DOWNLOAD_URL = "https://files.jrsoftware.org/is/6/innosetup-6.2.2.exe"

    def __init__(self, show_progress=True, install_path=None, download_url=None):
        """
        初始化类，设置是否显示进度条和输出信息、安装路径以及下载 URL
        :param show_progress: 是否显示进度条和输出信息
        :param install_path: 安装路径，若为 None 则使用默认路径
        :param download_url: 下载 URL，若为 None 则使用默认 URL
        """
        self.show_progress = show_progress
        self.install_path = install_path
        self.download_url = download_url if download_url else self.DEFAULT_INNO_SETUP_DOWNLOAD_URL
        if not self.is_innosetup_installed():
            self.install()
        else:
            if self.show_progress:
                print("Inno Setup is already installed. Installation will be stopped.")

    def is_innosetup_installed(self):
        """
        检测 Inno Setup 是否已经安装
        :return: 如果安装返回 True，否则返回 False
        """
        try:
            # 打开注册表中 Inno Setup 的安装信息所在的键
            key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\Inno Setup 6_is1")
            # 尝试获取显示名称，若能获取到则表示安装了 Inno Setup
            winreg.QueryValueEx(key, "DisplayName")
            winreg.CloseKey(key)
            return True
        except (FileNotFoundError, OSError):
            return False

    def download_file(self, url, save_path):
        """
        下载文件并显示彩色进度条
        :param url: 文件下载链接
        :param save_path: 保存文件的路径
        """
        response = requests.get(url, stream=True)
        total_size = int(response.headers.get('content-length', 0))
        block_size = 1024  # 1 Kibibyte
        if self.show_progress:
            # 自定义进度条样式，这里设置进度条颜色为绿色
            progress = Progress(
                TextColumn("[progress.description]{task.description}"),
                BarColumn(style="green"),
                "[progress.percentage]{task.percentage:>3.0f}%",
                "•",
                TransferSpeedColumn(),
                "•",
                TimeRemainingColumn(),
            )
            with progress:
                task = progress.add_task("[cyan]Downloading...", total=total_size)
                with open(save_path, 'wb') as file:
                    for data in response.iter_content(block_size):
                        file.write(data)
                        progress.update(task, advance=len(data))
        else:
            with open(save_path, 'wb') as file:
                for data in response.iter_content(block_size):
                    file.write(data)

        if total_size != 0:
            file_size = os.path.getsize(save_path)
            if file_size != total_size:
                print("Download error: File size does not match.")

    def install_innosetup(self, installer_path):
        """
        安装 Inno Setup
        :param installer_path: 安装程序的路径
        """
        try:
            if self.show_progress:
                print("Starting to install Inno Setup...")
            install_args = [installer_path, "/VERYSILENT"]
            if self.install_path:
                install_args.extend(["/DIR", self.install_path])
            # 执行安装程序
            subprocess.run(install_args, check=True)
            if self.show_progress:
                print("Inno Setup installed successfully.")
        except subprocess.CalledProcessError as e:
            if self.show_progress:
                print(f"Installation error: {e}")

    def install(self):
        """
        自动下载并安装 Inno Setup
        """
        # 下载路径
        download_path = os.path.join(os.getcwd(), "innosetup.exe")
        if self.show_progress:
            print("Starting to download the Inno Setup installer...")
        # 下载 Inno Setup 安装程序
        self.download_file(self.download_url, download_path)

        # 安装 Inno Setup
        if os.path.exists(download_path):
            self.install_innosetup(download_path)
            # 安装完成后删除下载的安装程序
            os.remove(download_path)
        else:
            if self.show_progress:
                print("The downloaded installer file does not exist. Installation cannot proceed.")


import ctypes
from ctypes import wintypes
from PIL import Image
import io


class DLLImageProcessor:
    def __init__(self, dll_path, resource_name='IMAGE_NAME', resource_type='PNG', output_path='processed_image.png'):
        self.dll_path = dll_path
        self.resource_name = resource_name
        self.resource_type = resource_type
        self.output_path = output_path

        # 定义 Windows API 函数
        self.LoadLibrary = ctypes.windll.kernel32.LoadLibraryW
        self.LoadLibrary.argtypes = [wintypes.LPCWSTR]
        self.LoadLibrary.restype = wintypes.HMODULE

        self.FindResource = ctypes.windll.kernel32.FindResourceW
        self.FindResource.argtypes = [wintypes.HMODULE, wintypes.LPCWSTR, wintypes.LPCWSTR]
        self.FindResource.restype = wintypes.HRSRC

        self.SizeofResource = ctypes.windll.kernel32.SizeofResource
        self.SizeofResource.argtypes = [wintypes.HMODULE, wintypes.HRSRC]
        self.SizeofResource.restype = wintypes.DWORD

        self.LoadResource = ctypes.windll.kernel32.LoadResource
        self.LoadResource.argtypes = [wintypes.HMODULE, wintypes.HRSRC]
        self.LoadResource.restype = wintypes.HGLOBAL

        self.LockResource = ctypes.windll.kernel32.LockResource
        self.LockResource.argtypes = [wintypes.HGLOBAL]
        self.LockResource.restype = ctypes.c_void_p

    def extract_image_from_dll(self):
        # 加载 DLL 文件
        hModule = self.LoadLibrary(self.dll_path)
        if hModule == 0:
            print("Failed to load DLL.")
            return None

        # 查找图片资源
        hResInfo = self.FindResource(hModule, self.resource_name, self.resource_type)
        if hResInfo == 0:
            print("Failed to find resource.")
            return None

        # 获取资源大小
        resource_size = self.SizeofResource(hModule, hResInfo)

        # 加载资源
        hGlobal = self.LoadResource(hModule, hResInfo)

        # 锁定资源
        lpResource = self.LockResource(hGlobal)

        # 读取资源数据
        resource_data = ctypes.string_at(lpResource, resource_size)

        return resource_data

    def process_and_save_image(self, image_data, resize=None, convert_to_gray=False):
        try:
            # 将二进制数据转换为 Pillow 的 Image 对象
            image = Image.open(io.BytesIO(image_data))

            # 调整图片大小
            if resize:
                image = image.resize(resize, Image.Resampling.LANCZOS)

            # 转换为灰度图
            if convert_to_gray:
                image = image.convert('L')

            # 保存图片
            image.save(self.output_path)
            print(f"Image saved to {self.output_path}")
        except Exception as e:
            print(f"Error processing or saving image: {e}")

    def run(self, resize=None, convert_to_gray=False):
        # 提取图片数据
        image_data = self.extract_image_from_dll()
        if image_data:
            # 处理并保存图片
            self.process_and_save_image(image_data, resize=resize, convert_to_gray=convert_to_gray)


from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='hyyap',
    version='0.0.5',
    description='hyyap',
    long_description=long_description,
    author='huang yi yi',
    author_email='363766687@qq.com',
    packages=find_packages(),
    package_data={
        "hyyap": ['*.exe'],
    },
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'hyy=hyyap._hyymain:main', 
        ],
    },
    python_requires='>=3.6',
    install_requires=[
        "tqdm",
        "rich",
        "requests",
        "py7zr",
        "pipreqs",
        "python-pptx",
        "openpyxl",
        "python-docx",
        "beautifulsoup4",
        "pillow",
        "pynvml",
        "wmi",
        "pywin32",
        "psutil",
        "certifi",
    ],
)


from fontTools.ttLib import TTFont, TTCollection
from fontTools.subset import Subsetter, Options
from fontTools.pens.recordingPen import RecordingPen
from fontTools.pens.transformPen import TransformPen


class TTFFontHandler:
    def __init__(self, path):
        """
        初始化 TTFFontHandler 类，加载指定路径的 .ttf 字体文件。

        :param path: .ttf 字体文件的路径
        """
        self.path = path
        self.font = None
        try:
            self.font = TTFont(path)
        except Exception as e:
            print(f"加载 .ttf 字体文件出错: {e}")

    def info(self):
        """获取 .ttf 字体基本信息"""
        if self.font:
            try:
                name = self.font['name'].getName(4, 3, 1, 1033).toUnicode()
                glyph_count = len(self.font.getGlyphSet())
                return name, glyph_count
            except Exception as e:
                print(f"获取 .ttf 字体信息出错: {e}")
        return None, None

    def unicodes(self, num=10):
        """获取 .ttf 字体前 num 个字符 Unicode 编码"""
        if self.font:
            try:
                unicodes = []
                for table in self.font['cmap'].tables:
                    for code, _ in table.cmap.items():
                        unicodes.append(code)
                return unicodes[:num]
            except Exception as e:
                print(f"获取 .ttf 字体 Unicode 编码出错: {e}")
        return []

    def mod_copyright(self, new_copyright):
        """修改 .ttf 字体版权信息"""
        if self.font:
            try:
                for record in self.font['name'].names:
                    if record.nameID == 0:
                        record.string = new_copyright.encode('utf-16-be')
                self.font.save(self.path)
                return True
            except Exception as e:
                print(f"修改 .ttf 字体版权信息出错: {e}")
        return False

    def subset(self, output_path, unicodes):
        """对 .ttf 字体进行子集化"""
        if self.font:
            try:
                options = Options()
                subsetter = Subsetter(options=options)
                subsetter.populate(unicodes=unicodes)
                subsetter.subset(self.font)
                self.font.save(output_path)
                return True
            except Exception as e:
                print(f"子集化 .ttf 字体出错: {e}")
        return False

    def units_per_em(self):
        """获取 .ttf 字体每 em 单位数"""
        if self.font:
            try:
                return self.font['head'].unitsPerEm
            except Exception as e:
                print(f"获取 .ttf 每 em 单位数出错: {e}")
        return None

    def ascender(self):
        """获取 .ttf 字体上伸部高度"""
        if self.font:
            try:
                return self.font['hhea'].ascender
            except Exception as e:
                print(f"获取 .ttf 上伸部高度出错: {e}")
        return None

    def descender(self):
        """获取 .ttf 字体下伸部高度"""
        if self.font:
            try:
                return self.font['hhea'].descender
            except Exception as e:
                print(f"获取 .ttf 下伸部高度出错: {e}")
        return None

    def line_gap(self):
        """获取 .ttf 字体行间距"""
        if self.font:
            try:
                return self.font['hhea'].lineGap
            except Exception as e:
                print(f"获取 .ttf 行间距出错: {e}")
        return None

    def glyph_width(self, glyph_name):
        """获取 .ttf 字体指定字形宽度"""
        if self.font:
            try:
                return self.font['hmtx'][glyph_name][0]
            except Exception as e:
                print(f"获取 .ttf 字形宽度出错: {e}")
        return None

    def glyph_height(self, glyph_name):
        """获取 .ttf 字体指定字形高度"""
        if self.font:
            try:
                glyph_set = self.font.getGlyphSet()
                glyph = glyph_set[glyph_name]
                pen = RecordingPen()
                glyph.draw(pen)
                bounds = pen.value.getBounds()
                if bounds:
                    return bounds[3] - bounds[1]
            except Exception as e:
                print(f"获取 .ttf 字形高度出错: {e}")
        return None

    def glyph_bbox(self, glyph_name):
        """获取 .ttf 字体指定字形边界框"""
        if self.font:
            try:
                glyph_set = self.font.getGlyphSet()
                glyph = glyph_set[glyph_name]
                pen = RecordingPen()
                glyph.draw(pen)
                return pen.value.getBounds()
            except Exception as e:
                print(f"获取 .ttf 字形边界框出错: {e}")
        return None

    def scale_glyph(self, glyph_name, scale_x, scale_y, output_path):
        """缩放 .ttf 字体指定字形"""
        if self.font:
            try:
                glyph_set = self.font.getGlyphSet()
                glyph = glyph_set[glyph_name]
                pen = RecordingPen()
                glyph.draw(pen)
                transform = (scale_x, 0, 0, scale_y, 0, 0)
                tpen = TransformPen(pen, transform)
                glyph.draw(tpen)
                new_glyph = glyph_set.fromPen(tpen.value)
                self.font['glyf'][glyph_name] = new_glyph
                self.font.save(output_path)
                return True
            except Exception as e:
                print(f"缩放 .ttf 字形出错: {e}")
        return False

    def skew_glyph(self, glyph_name, skew_x, skew_y, output_path):
        """倾斜 .ttf 字体指定字形"""
        if self.font:
            try:
                glyph_set = self.font.getGlyphSet()
                glyph = glyph_set[glyph_name]
                pen = RecordingPen()
                glyph.draw(pen)
                transform = (1, skew_x, skew_y, 1, 0, 0)
                tpen = TransformPen(pen, transform)
                glyph.draw(tpen)
                new_glyph = glyph_set.fromPen(tpen.value)
                self.font['glyf'][glyph_name] = new_glyph
                self.font.save(output_path)
                return True
            except Exception as e:
                print(f"倾斜 .ttf 字形出错: {e}")
        return False

    def move_glyph(self, glyph_name, dx, dy, output_path):
        """平移 .ttf 字体指定字形"""
        if self.font:
            try:
                glyph_set = self.font.getGlyphSet()
                glyph = glyph_set[glyph_name]
                pen = RecordingPen()
                glyph.draw(pen)
                transform = (1, 0, 0, 1, dx, dy)
                tpen = TransformPen(pen, transform)
                glyph.draw(tpen)
                new_glyph = glyph_set.fromPen(tpen.value)
                self.font['glyf'][glyph_name] = new_glyph
                self.font.save(output_path)
                return True
            except Exception as e:
                print(f"平移 .ttf 字形出错: {e}")
        return False

    def set_version(self, version):
        """设置 .ttf 字体版本号"""
        if self.font:
            try:
                self.font['head'].fontRevision = float(version)
                self.font.save(self.path)
                return True
            except Exception as e:
                print(f"设置 .ttf 版本号出错: {e}")
        return False

    def get_version(self):
        """获取 .ttf 字体版本号"""
        if self.font:
            try:
                return self.font['head'].fontRevision
            except Exception as e:
                print(f"获取 .ttf 版本号出错: {e}")
        return None

    def set_mac_style(self, bold=False, italic=False):
        """设置 .ttf 字体 Mac 样式"""
        if self.font:
            try:
                mac_style = 0
                if bold:
                    mac_style |= 1
                if italic:
                    mac_style |= 2
                self.font['head'].macStyle = mac_style
                self.font.save(self.path)
                return True
            except Exception as e:
                print(f"设置 .ttf Mac 样式出错: {e}")
        return False

    def get_mac_style(self):
        """获取 .ttf 字体 Mac 样式"""
        if self.font:
            try:
                return self.font['head'].macStyle
            except Exception as e:
                print(f"获取 .ttf Mac 样式出错: {e}")
        return None

    def set_fs_selection(self, bold=False, italic=False, underline=False):
        """设置 .ttf 字体 fsSelection 字段"""
        if self.font:
            try:
                fs_selection = 0
                if bold:
                    fs_selection |= 0x01
                if italic:
                    fs_selection |= 0x02
                if underline:
                    fs_selection |= 0x04
                self.font['OS/2'].fsSelection = fs_selection
                self.font.save(self.path)
                return True
            except Exception as e:
                print(f"设置 .ttf fsSelection 字段出错: {e}")
        return False

    def get_fs_selection(self):
        """获取 .ttf 字体 fsSelection 字段"""
        if self.font:
            try:
                return self.font['OS/2'].fsSelection
            except Exception as e:
                print(f"获取 .ttf fsSelection 字段出错: {e}")
        return None

    def close(self):
        """关闭 .ttf 字体文件"""
        if self.font:
            self.font.close()


class TTCFontHandler:
    def __init__(self, path):
        """
        初始化 TTCFontHandler 类，加载指定路径的 .ttc 字体集合文件。

        :param path: .ttc 字体集合文件的路径
        """
        self.path = path
        self.ttc = None
        try:
            self.ttc = TTCollection(path)
        except Exception as e:
            print(f"加载 .ttc 字体集合文件出错: {e}")

    def names(self):
        """
        获取 .ttc 字体集合中各字体名称。

        :return: 包含各字体名称的列表，出错则返回空列表
        """
        if self.ttc:
            try:
                return [font['name'].getName(4, 3, 1, 1033).toUnicode() for font in self.ttc.fonts]
            except Exception as e:
                print(f"获取 .ttc 字体名称出错: {e}")
        return []

    def extract(self, index=0, output_path='extracted_font.ttf'):
        """
        从 .ttc 中提取指定索引字体保存为 .ttf。

        :param index: 要提取的字体索引，默认为 0
        :param output_path: 提取后保存的 .ttf 文件路径，默认为 'extracted_font.ttf'
        :return: 提取成功返回 True，失败返回 False
        """
        if self.ttc:
            try:
                if 0 <= index < len(self.ttc.fonts):
                    self.ttc.fonts[index].save(output_path)
                    return True
                print("索引超出 .ttc 字体集合范围")
            except Exception as e:
                print(f"提取 .ttc 字体出错: {e}")
        return False

    def info(self, index=0):
        """
        获取 .ttc 中指定索引字体基本信息。

        :param index: 字体索引，默认为 0
        :return: 包含字体名称和字符集数量的元组，出错则返回 (None, None)
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                font = self.ttc.fonts[index]
                name = font['name'].getName(4, 3, 1, 1033).toUnicode()
                glyph_count = len(font.getGlyphSet())
                return name, glyph_count
            except Exception as e:
                print(f"获取 .ttc 字体信息出错: {e}")
        return None, None

    def unicodes(self, index=0, num=10):
        """
        获取 .ttc 中指定索引字体前 num 个字符 Unicode 编码。

        :param index: 字体索引，默认为 0
        :param num: 要获取的字符数量，默认为 10
        :return: 包含前 num 个字符 Unicode 编码的列表，出错则返回空列表
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                font = self.ttc.fonts[index]
                unicodes = []
                for table in font['cmap'].tables:
                    for code, _ in table.cmap.items():
                        unicodes.append(code)
                return unicodes[:num]
            except Exception as e:
                print(f"获取 .ttc 字体 Unicode 编码出错: {e}")
        return []

    def mod_copyright(self, index=0, new_copyright=""):
        """
        修改 .ttc 中指定索引字体版权信息。

        :param index: 字体索引，默认为 0
        :param new_copyright: 新的版权信息
        :return: 修改成功返回 True，失败返回 False
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                font = self.ttc.fonts[index]
                for record in font['name'].names:
                    if record.nameID == 0:
                        record.string = new_copyright.encode('utf-16-be')
                font.save(self.path)
                return True
            except Exception as e:
                print(f"修改 .ttc 字体版权信息出错: {e}")
        return False

    def subset(self, index=0, output_path='subset_font.ttf', unicodes=[]):
        """
        对 .ttc 中指定索引字体进行子集化。

        :param index: 字体索引，默认为 0
        :param output_path: 子集化后保存的 .ttf 文件路径，默认为 'subset_font.ttf'
        :param unicodes: 要保留的 Unicode 字符列表
        :return: 子集化成功返回 True，失败返回 False
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                font = self.ttc.fonts[index]
                options = Options()
                subsetter = Subsetter(options=options)
                subsetter.populate(unicodes=unicodes)
                subsetter.subset(font)
                font.save(output_path)
                return True
            except Exception as e:
                print(f"子集化 .ttc 字体出错: {e}")
        return False

    def units_per_em(self, index=0):
        """
        获取 .ttc 中指定索引字体每 em 单位数。

        :param index: 字体索引，默认为 0
        :return: 每 em 单位数，出错则返回 None
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                return self.ttc.fonts[index]['head'].unitsPerEm
            except Exception as e:
                print(f"获取 .ttc 每 em 单位数出错: {e}")
        return None

    def ascender(self, index=0):
        """
        获取 .ttc 中指定索引字体上伸部高度。

        :param index: 字体索引，默认为 0
        :return: 上伸部高度，出错则返回 None
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                return self.ttc.fonts[index]['hhea'].ascender
            except Exception as e:
                print(f"获取 .ttc 上伸部高度出错: {e}")
        return None

    def descender(self, index=0):
        """
        获取 .ttc 中指定索引字体下伸部高度。

        :param index: 字体索引，默认为 0
        :return: 下伸部高度，出错则返回 None
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                return self.ttc.fonts[index]['hhea'].descender
            except Exception as e:
                print(f"获取 .ttc 下伸部高度出错: {e}")
        return None

    def line_gap(self, index=0):
        """
        获取 .ttc 中指定索引字体行间距。

        :param index: 字体索引，默认为 0
        :return: 行间距，出错则返回 None
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                return self.ttc.fonts[index]['hhea'].lineGap
            except Exception as e:
                print(f"获取 .ttc 行间距出错: {e}")
        return None

    def glyph_width(self, index=0, glyph_name=""):
        """
        获取 .ttc 中指定索引字体指定字形宽度。

        :param index: 字体索引，默认为 0
        :param glyph_name: 字形名称
        :return: 字形宽度，出错则返回 None
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                return self.ttc.fonts[index]['hmtx'][glyph_name][0]
            except Exception as e:
                print(f"获取 .ttc 字形宽度出错: {e}")
        return None

    def glyph_height(self, index=0, glyph_name=""):
        """
        获取 .ttc 中指定索引字体指定字形高度。

        :param index: 字体索引，默认为 0
        :param glyph_name: 字形名称
        :return: 字形高度，出错则返回 None
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                font = self.ttc.fonts[index]
                glyph_set = font.getGlyphSet()
                glyph = glyph_set[glyph_name]
                pen = RecordingPen()
                glyph.draw(pen)
                bounds = pen.value.getBounds()
                if bounds:
                    return bounds[3] - bounds[1]
            except Exception as e:
                print(f"获取 .ttc 字形高度出错: {e}")
        return None

    def glyph_bbox(self, index=0, glyph_name=""):
        """
        获取 .ttc 中指定索引字体指定字形边界框。

        :param index: 字体索引，默认为 0
        :param glyph_name: 字形名称
        :return: 字形边界框，出错则返回 None
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                font = self.ttc.fonts[index]
                glyph_set = font.getGlyphSet()
                glyph = glyph_set[glyph_name]
                pen = RecordingPen()
                glyph.draw(pen)
                return pen.value.getBounds()
            except Exception as e:
                print(f"获取 .ttc 字形边界框出错: {e}")
        return None

    def scale_glyph(self, index=0, glyph_name="", scale_x=1, scale_y=1, output_path='scaled_font.ttf'):
        """
        缩放 .ttc 中指定索引字体指定字形。

        :param index: 字体索引，默认为 0
        :param glyph_name: 字形名称
        :param scale_x: x 方向缩放比例，默认为 1
        :param scale_y: y 方向缩放比例，默认为 1
        :param output_path: 处理后保存的 .ttf 文件路径，默认为 'scaled_font.ttf'
        :return: 缩放成功返回 True，失败返回 False
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                font = self.ttc.fonts[index]
                glyph_set = font.getGlyphSet()
                glyph = glyph_set[glyph_name]
                pen = RecordingPen()
                glyph.draw(pen)
                transform = (scale_x, 0, 0, scale_y, 0, 0)
                tpen = TransformPen(pen, transform)
                glyph.draw(tpen)
                new_glyph = glyph_set.fromPen(tpen.value)
                font['glyf'][glyph_name] = new_glyph
                font.save(output_path)
                return True
            except Exception as e:
                print(f"缩放 .ttc 字形出错: {e}")
        return False

    def skew_glyph(self, index=0, glyph_name="", skew_x=0, skew_y=0, output_path='skewed_font.ttf'):
        """
        倾斜 .ttc 中指定索引字体指定字形。

        :param index: 字体索引，默认为 0
        :param glyph_name: 字形名称
        :param skew_x: x 方向倾斜比例，默认为 0
        :param skew_y: y 方向倾斜比例，默认为 0
        :param output_path: 处理后保存的 .ttf 文件路径，默认为 'skewed_font.ttf'
        :return: 倾斜成功返回 True，失败返回 False
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                font = self.ttc.fonts[index]
                glyph_set = font.getGlyphSet()
                glyph = glyph_set[glyph_name]
                pen = RecordingPen()
                glyph.draw(pen)
                transform = (1, skew_x, skew_y, 1, 0, 0)
                tpen = TransformPen(pen, transform)
                glyph.draw(tpen)
                new_glyph = glyph_set.fromPen(tpen.value)
                font['glyf'][glyph_name] = new_glyph
                font.save(output_path)
                return True
            except Exception as e:
                print(f"倾斜 .ttc 字形出错: {e}")
        return False

    def move_glyph(self, index=0, glyph_name="", dx=0, dy=0, output_path='moved_font.ttf'):
        """
        平移 .ttc 中指定索引字体指定字形。

        :param index: 字体索引，默认为 0
        :param glyph_name: 字形名称
        :param dx: x 方向平移量，默认为 0
        :param dy: y 方向平移量，默认为 0
        :param output_path: 处理后保存的 .ttf 文件路径，默认为 'moved_font.ttf'
        :return: 平移成功返回 True，失败返回 False
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                font = self.ttc.fonts[index]
                glyph_set = font.getGlyphSet()
                glyph = glyph_set[glyph_name]
                pen = RecordingPen()
                glyph.draw(pen)
                transform = (1, 0, 0, 1, dx, dy)
                tpen = TransformPen(pen, transform)
                glyph.draw(tpen)
                new_glyph = glyph_set.fromPen(tpen.value)
                font['glyf'][glyph_name] = new_glyph
                font.save(output_path)
                return True
            except Exception as e:
                print(f"平移 .ttc 字形出错: {e}")
        return False

    def set_version(self, index=0, version=1.0):
        """
        设置 .ttc 中指定索引字体版本号。

        :param index: 字体索引，默认为 0
        :param version: 版本号，默认为 1.0
        :return: 设置成功返回 True，失败返回 False
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                font = self.ttc.fonts[index]
                font['head'].fontRevision = float(version)
                font.save(self.path)
                return True
            except Exception as e:
                print(f"设置 .ttc 版本号出错: {e}")
        return False

    def version(self, index=0):
        """
        获取 .ttc 中指定索引字体版本号。

        :param index: 字体索引，默认为 0
        :return: 版本号，出错则返回 None
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                return self.ttc.fonts[index]['head'].fontRevision
            except Exception as e:
                print(f"获取 .ttc 版本号出错: {e}")
        return None

    def mac_style(self, index=0, bold=False, italic=False):
        """
        设置 .ttc 中指定索引字体 Mac 样式。

        :param index: 字体索引，默认为 0
        :param bold: 是否为粗体，默认为 False
        :param italic: 是否为斜体，默认为 False
        :return: 设置成功返回 True，失败返回 False
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                font = self.ttc.fonts[index]
                mac_style = 0
                if bold:
                    mac_style |= 1
                if italic:
                    mac_style |= 2
                font['head'].macStyle = mac_style
                font.save(self.path)
                return True
            except Exception as e:
                print(f"设置 .ttc Mac 样式出错: {e}")
        return False

    def mac_style(self, index=0):
        """
        获取 .ttc 中指定索引字体 Mac 样式。

        :param index: 字体索引，默认为 0
        :return: Mac 样式，出错则返回 None
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                return self.ttc.fonts[index]['head'].macStyle
            except Exception as e:
                print(f"获取 .ttc Mac 样式出错: {e}")
        return None

    def fs_selection(self, index=0, bold=False, italic=False, underline=False):
        """
        设置 .ttc 中指定索引字体的 fsSelection 字段。

        :param index: 字体在 .ttc 集合中的索引，默认为 0
        :param bold: 是否设置为粗体，默认为 False
        :param italic: 是否设置为斜体，默认为 False
        :param underline: 是否设置有下划线，默认为 False
        :return: 设置成功返回 True，失败返回 False
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                font = self.ttc.fonts[index]
                fs_selection = 0
                if bold:
                    fs_selection |= 0x01
                if italic:
                    fs_selection |= 0x02
                if underline:
                    fs_selection |= 0x04
                font['OS/2'].fsSelection = fs_selection
                font.save(self.path)
                return True
            except Exception as e:
                print(f"设置 .ttc 字体 fsSelection 字段出错: {e}")
        return False

    def fs_selection(self, index=0):
        """
        获取 .ttc 中指定索引字体的 fsSelection 字段。

        :param index: 字体在 .ttc 集合中的索引，默认为 0
        :return: fsSelection 字段的值，出错则返回 None
        """
        if self.ttc and 0 <= index < len(self.ttc.fonts):
            try:
                return self.ttc.fonts[index]['OS/2'].fsSelection
            except Exception as e:
                print(f"获取 .ttc 字体 fsSelection 字段出错: {e}")
        return None

    def close(self):
        """
        关闭相关资源，尝试释放可能占用的资源。
        """
        try:

            if self.ttc:
                self.ttc = None
        except Exception as e:
            print(f"Error: {e}")



from docx import Document
from docx.shared import Pt, Inches, Cm
from docx.enum.text import WD_ALIGN_PARAGRAPH
from docx.enum.table import WD_ALIGN_VERTICAL


class WordDocx:
    def __init__(self):
        self.doc = Document()

    def title(self, title_text, level=1, font_size=20):
        """向文档中添加标题"""
        title = self.doc.add_heading(title_text, level=level)
        title.runs[0].font.size = Pt(font_size)
        return self

    def paragraph(self, paragraph_text, font_size=12):
        """向文档中添加段落"""
        paragraph = self.doc.add_paragraph(paragraph_text)
        paragraph.runs[0].font.size = Pt(font_size)
        return self

    def bullet_list(self, items):
        """向文档中添加无序列表"""
        for item in items:
            self.doc.add_paragraph(item, style='List Bullet')
        return self

    def numbered_list(self, items):
        """向文档中添加有序列表"""
        for item in items:
            self.doc.add_paragraph(item, style='List Number')
        return self

    def picture(self, picture_path, width=4):
        """向文档中添加图片"""
        self.doc.add_picture(picture_path, width=Inches(width))
        return self

    def table(self, rows, cols, headers, data):
        """向文档中添加表格"""
        table = self.doc.add_table(rows=rows, cols=cols)
        hdr_cells = table.rows[0].cells
        for i, header in enumerate(headers):
            hdr_cells[i].text = header
        for row_num, row_data in enumerate(data):
            row_cells = table.rows[row_num + 1].cells
            for col_num, cell_data in enumerate(row_data):
                row_cells[col_num].text = cell_data
        return self

    def table_column_widths(self, table_index, column_widths):
        """设置表格指定列的宽度"""
        if table_index < len(self.doc.tables):
            table = self.doc.tables[table_index]
            for col_index, width in enumerate(column_widths):
                if col_index < len(table.columns):
                    table.columns[col_index].width = Cm(width)
        return self

    def table_row_height(self, table_index, row_index, line_spacing=2.0):
        """设置表格指定行的行高（通过段落行距调整）"""
        if table_index < len(self.doc.tables):
            table = self.doc.tables[table_index]
            if row_index < len(table.rows):
                for cell in table.rows[row_index].cells:
                    paragraph = cell.paragraphs[0]
                    paragraph_format = paragraph.paragraph_format
                    paragraph_format.line_spacing = line_spacing
                    cell.vertical_alignment = WD_ALIGN_VERTICAL.CENTER
                    paragraph.alignment = WD_ALIGN_PARAGRAPH.CENTER
        return self

    def save(self, file_path):
        """保存文档，确保保存为 .docx 格式"""
        if not file_path.lower().endswith('.docx'):
            if '.' in file_path:
                file_path = file_path.rsplit('.', 1)[0] + '.docx'
            else:
                file_path = file_path + '.docx'
        self.doc.save(file_path)

from openpyxl import Workbook
from openpyxl.styles import Font
from openpyxl.utils import get_column_letter


class ExcelXlsx:
    def __init__(self):
        # 初始化一个新的 Excel 工作簿
        self.wb = Workbook()
        # 获取默认的工作表
        self.ws = self.wb.active

    def add_title(self, title_text, row=1, col=1):
        """向 Excel 工作表中添加标题"""
        cell = self.ws.cell(row=row, column=col, value=title_text)
        cell.font = Font(size=20, bold=True)
        return self

    def add_paragraph(self, paragraph_text, row, col):
        """向 Excel 工作表中添加段落文本"""
        cell = self.ws.cell(row=row, column=col, value=paragraph_text)
        cell.font = Font(size=12)
        return self

    def add_list(self, items, start_row, col, is_bullet=True):
        """向 Excel 工作表中添加列表"""
        for i, item in enumerate(items, start=start_row):
            cell = self.ws.cell(row=i, column=col)
            if is_bullet:
                cell.value = f"• {item}"
            else:
                cell.value = f"{i - start_row + 1}. {item}"
            cell.font = Font(size=12)
        return self

    def add_picture(self):
        # 注意：openpyxl 对图片操作相对复杂，这里暂不实现，可考虑使用其他库如 pandas 结合 xlsxwriter
        print("Excel 中添加图片暂不支持此简单实现。")
        return self

    def add_table(self, headers, data, start_row, start_col):
        """向 Excel 工作表中添加表格"""
        # 写入表头
        for col_num, header in enumerate(headers, start=start_col):
            cell = self.ws.cell(row=start_row, column=col_num)
            cell.value = header
            cell.font = Font(size=12, bold=True)
        # 写入表格数据
        for row_num, row_data in enumerate(data, start=start_row + 1):
            for col_num, cell_data in enumerate(row_data, start=start_col):
                cell = self.ws.cell(row=row_num, column=col_num)
                cell.value = cell_data
        return self

    def set_table_column_widths(self, start_col, column_widths):
        """设置表格指定列的宽度"""
        for col_index, width in enumerate(column_widths, start=start_col):
            col_letter = get_column_letter(col_index)
            self.ws.column_dimensions[col_letter].width = width
        return self

    def set_table_row_height(self, start_row, row_indexes, heights):
        """设置表格指定行的行高"""
        for i, row_index in enumerate(row_indexes, start=start_row):
            self.ws.row_dimensions[row_index].height = heights[i]
        return self

    def save(self, file_path):
        """保存 Excel 文档，确保保存为 .xlsx 格式"""
        if not file_path.lower().endswith('.xlsx'):
            if '.' in file_path:
                file_path = file_path.rsplit('.', 1)[0] + '.xlsx'
            else:
                file_path = file_path + '.xlsx'
        self.wb.save(file_path)

from pptx import Presentation
from pptx.util import Inches, Pt
from pptx.enum.text import PP_ALIGN


class PPTGenerator:
    def __init__(self):
        # 初始化一个新的 PPT 演示文稿
        self.prs = Presentation()

    def add_title_slide(self, title_text, subtitle_text=""):
        """添加标题幻灯片"""
        title_slide_layout = self.prs.slide_layouts[0]
        slide = self.prs.slides.add_slide(title_slide_layout)
        title = slide.shapes.title
        subtitle = slide.placeholders[1]
        title.text = title_text
        subtitle.text = subtitle_text
        return self

    def add_content_slide(self, title_text, content_items):
        """添加带有内容列表的幻灯片"""
        content_slide_layout = self.prs.slide_layouts[1]
        slide = self.prs.slides.add_slide(content_slide_layout)
        title = slide.shapes.title
        body_shape = slide.shapes.placeholders[1]
        title.text = title_text
        tf = body_shape.text_frame
        for item in content_items:
            p = tf.add_paragraph()
            p.text = item
            p.font.size = Pt(18)
        return self

    def add_table_slide(self, title_text, headers, data):
        """添加带有表格的幻灯片"""
        table_slide_layout = self.prs.slide_layouts[5]
        slide = self.prs.slides.add_slide(table_slide_layout)
        title = slide.shapes.title
        title.text = title_text
        rows, cols = len(data) + 1, len(headers)
        left = top = Inches(2)
        width = Inches(6)
        height = Inches(0.8)
        table = slide.shapes.add_table(rows, cols, left, top, width, height).table
        # 写入表头
        for col_index, header in enumerate(headers):
            table.cell(0, col_index).text = header
            table.cell(0, col_index).text_frame.paragraphs[0].alignment = PP_ALIGN.CENTER
        # 写入表格数据
        for row_index, row_data in enumerate(data, start=1):
            for col_index, cell_data in enumerate(row_data):
                table.cell(row_index, col_index).text = cell_data
        return self

    def add_picture_slide(self, title_text, picture_path):
        """添加带有图片的幻灯片"""
        picture_slide_layout = self.prs.slide_layouts[5]
        slide = self.prs.slides.add_slide(picture_slide_layout)
        title = slide.shapes.title
        title.text = title_text
        left = top = Inches(2)
        slide.shapes.add_picture(picture_path, left, top, width=Inches(6))
        return self

    def save(self, file_path):
        """保存 PPT 文档，确保保存为 .pptx 格式"""
        if not file_path.lower().endswith('.pptx'):
            if '.' in file_path:
                file_path = file_path.rsplit('.', 1)[0] + '.pptx'
            else:
                file_path = file_path + '.pptx'
        self.prs.save(file_path)